function [Ti, Tf, Ri, Rf, kiz, kfz, ai, af, tthi, tthf, qx, qy, qp, qz,D] = GISAXS_wave_Amp_Vector2(ai, af, tthi, tthf, lambda, eden, beta, zinterface, roughness)
% [Ti, Tf, Ri, Rf, kiz, kfz] = GISAXS_wave_Amp_Vector(ai, af, tthi, tthf,
% lambda, eden, beta, zinterface, roughness, symofAsym, layernumber)
% output arguments could be Ti, Tf, Ri, Rf, kiz, kfz, ai, af, tthi, tthf, 
% qx, qy, qp, qz, and D.
%
% There are two modes
% Mode 1 : [Ti, Tf, Ri, Rf, kiz, kfz] = GISAXS_wave_Amp_Vector3(ai, af, tthi, tthf, lambda, eden, beta, zinterface, roughness, [])
% Mode 2 : [Ti, Tf, Ri, Rf, kiz, kfz] = GISAXS_wave_Amp_Vector3(ai, af, tthi, tthf, lambda, eden, beta, zinterface, roughness, 1)
% Mode 1 is for the multilayer DWBA calculation.
% For instance, Ti of Mode 1 is a cell whose element number is that of eden (or number of layers)
% Ti of Mode 2 contains the wave amplitude at a specific layer (layernumber, which start from 1 to N)
% When the zposition is specified, one can
% determine the layernumber as follows
%    layernumber = find(zinterface <= zposition);
%    if isempty(t)
%        layernumber = length(zinterface)+1;
%    end
%    layernumber = layernumber(1);
% When you perform the following for the calculated Ti by Mode 1
%        k = size(Ti);b=permute(ti, [k(end),k(1:end-1)]);reshape(b(1,:), k(1:end-1));
% you can get the same results of Mode 2
%
% ai : incident angle, ex) ai = 0.1 or [0.1, 0.2, 0.3];
% af : exit angle, ex) af = 0.1 or [0.1, 0.2, 0.3];
% lambda = 1.68;
% eden : electron density of medium (length of eden = 1 + length of z(position of interface)
%       eden = [0, 0.35, 0.7]
% beta
% z : position of interfaces
%       z = [0, -500];
% roughnessgy
%       roughness = [10, 3];
% layernumber : index of the layer of interest > 0. ex) 1, 2...
% symorAsym, when symorAsym = 0 asym mode. when it is 1, sym mode.
%       sym mode is ai = af
% returns, Ti, Tf, Ri, Rf, kiz, kfz are the wave Amp and wave vector at the media existing at zposition....
% qz and D are cell arrays. Both qz and D have 4 columns.
%            D{i,1} = aTi.*aTf;
%            D{i,2} = aTi.*aRf;
%            D{i,3} = aRi.*aTf;
%            D{i,4} = aRi.*aRf;

% Dimensions of the result
% When length of tthf or tthi >0 and that of ai or af > 0
%       dim of qp and qz{1,1} is the same, 
% When length of tthf or tthi > 0 and that of ai and af == 1,
%       dim of qp is larger (qp is 1D, qz{1,1} is 0D)
% This is because qp should have one additional dimension when length of
%       tthi or tthf > 1
% 
% Byeongdu Lee, Sep. 30th, 2006
% identical to GISAXS_wave_Amp_Vector.m
% difference is that this m file does not expand af and tth space.
% the element numbers of af and tth should be the same.

r0 = 2.82E-5;
k0 = 2*pi/lambda;
d1 = 1/(2*pi)*r0*lambda^2*eden;
b1 = beta;
j = sqrt(-1);
nx = 1-d1+j*b1;
sig = roughness;

dimnx = length(nx);

% one more dimension for index of layers
[rri, tti, kkiz] = Xrefl(ai, nx,  zinterface, sig, lambda);
[rrf, ttf, kkfz] = Xrefl(af, nx,  zinterface, sig, lambda);
kkfz = -kkfz;   %because, incident beam will go down
ai = deg2rad(ai);
af = deg2rad(af);
tthi=deg2rad(tthi);
tthf=deg2rad(tthf);

    qx = k0*(cos(af).*cos(tthf)-cos(ai).*cos(tthi));
    qy = k0*(cos(af).*sin(tthf)-cos(ai).*sin(tthi));
    qp = sqrt(qx.^2+qy.^2);

    Ti = cell(dimnx, 1);
    Tf = cell(dimnx, 1);
    Ri = cell(dimnx, 1);
    Rf = cell(dimnx, 1);
    kiz = cell(dimnx, 1);
    kfz = cell(dimnx, 1);
    qz = cell(dimnx, 4);
    D = cell(dimnx, 4);

    for i=1:dimnx
        %[aTi, aTf, aRi, aRf, akiz, akfz] = expand_par(sizeai, sizeaf, rri, rrf, tti, ttf, kkiz, kkfz, i);
        Ti{i} = tti(:,i);
        Ri{i} = rri(:,i);
        Tf{i} = ttf(:,i);
        Rf{i} = rrf(:,i);
        kiz{i} = kkiz(:,i);
        kfz{i} = kkfz(:,i);

        if nargout > 10
            qz{i,1} = kfz{i}-kiz{i};
            qz{i,2} = -kfz{i}-kiz{i};
            qz{i,3} = kfz{i}+kiz{i};
            qz{i,4} = -kfz{i}+kiz{i};
            if i<dimnx
            %D{i,1} = aTi.*aTf.*exp(-j*qz{i,1}*zinterface(i));
            %D{i,2} = aTi.*aRf.*exp(-j*qz{i,2}*zinterface(i));
            %D{i,3} = aRi.*aTf.*exp(-j*qz{i,3}*zinterface(i));
            %D{i,4} = aRi.*aRf.*exp(-j*qz{i,4}*zinterface(i));
            D{i,1} = Ti{i}.*Tf{i};
            D{i,2} = Ti{i}.*Rf{i};
            D{i,3} = Ri{i}.*Tf{i};
            D{i,4} = Ri{i}.*Rf{i};
            end
        end
    end


function a = deg2rad(a)
a = a*pi/180;

