function [R, T, k] = Xrefl(ai, nx, z, sig, lambda)
% Usage: [R, T, k] = Xrefl(ai, nx, z, sig, lambda)
%       or [R, T, k] = Xrefl(ai, [eden, beta], z, sig, lambda)
% Referece: G. Renaud et al. Surface Science Reports 64 (2009), 255-380.
% The transfer matrix In page 276 can be rewritten as
% rjj1 = (kzj - kzj+1)/(kzj+kzj1)
% Xj = Rj/Tj
% Xj = e^(2 i kzj zj1) [rjj1 + e^(-2 i kzj1 zj1) Xj1] / [1+rjj1*e^(-2 i
% kzj1 zj1) Xj1]
%, where rjj1 = r_{j,(j+1)}, kzj = k_(z,j), kzj1 = k_{z, (j+1)}, Xj1 =
%  X_{j+1}.
%
% This will be solved with boundary condition of T_1 = 1 and R_N = 0.
% Index of layers: 1 ~ N
%   1 for vacuum
%   N for substrate.
% Interface : 2 ~ N
%   2 is the top surface.
%   N is the substrate surface.
%   z_1 is meaningless...
% Note that in other software, dwbaxr_old.m and parratt.m, z_N+1 is
% meaningless.....
% The origin of the coordinate is at top of the surface.
% z > 0 upward.
% 
% Also, note that for using this code...
% nx = 1-delta+i*beta
% 
% Byeongdu Lee

i = sqrt(-1);
r0 = 2.82E-5;k0 = 2*pi/lambda;
if size(nx, 2) == 2 % if nx = [eden, beta]
    if nx(1,1)<0.1 % then nx(:,1) is edensity and nx is not the refractive index.
        eden = nx(:,1);
        beta = nx(:,2);
        d1 = 1/(2*pi)*r0*lambda^2*eden;
        nx = 1-d1+i*beta;
    end
end

z = z(:)';
sig = sig(:)';

if length(z) == length(nx)
    if z(1) == 0
        z = [z(end), z(1:end-1)];
        if ~isempty(sig)
            sig = [sig(end), sig(1:end-1)];
        end
    end
elseif length(z) == length(nx)-1
    z = [-1, z];
    if ~isempty(sig)
        sig = [-1, sig];
    end
end

n = length(nx); % Number of layers.

a = ai;

% Calculate wavevectors for n layers.
k = zeros(numel(a), n);
for iLayer=1:n
    temp = -k0*sqrt(nx(iLayer).^2 - cos(deg2rad(a(:))).^2);
    k(:, iLayer) = reshape(temp, length(temp), 1);
end

% Calculate r_{j,j+1} and matrix elements..
rjj1 = zeros(numel(a), n);
pjj1 = rjj1;
mjj1 = rjj1;

for iLayer = 1:(n-1)
    rjj1(:, iLayer) = (k(:, iLayer)-k(:, iLayer+1))./(k(:, iLayer)+k(:, iLayer+1));
    if ~isempty(sig)
        rjj1(:, iLayer) = rjj1(:, iLayer).*exp(-2.*k(:, iLayer).*k(:, iLayer+1)*sig(iLayer+1)^2);   % gaussian roughness. M. Tolan 2.34)
    end
    pjj1(:, iLayer) = (k(:, iLayer)+k(:, iLayer+1))./(2*k(:, iLayer));
    mjj1(:, iLayer) = (k(:, iLayer)-k(:, iLayer+1))./(2*k(:, iLayer));
    if ~isempty(sig)
        pjj1(:, iLayer) = pjj1(:, iLayer).*exp(-i*(k(:, iLayer) + k(:, iLayer+1))*sig(iLayer+1));
        mjj1(:, iLayer) = mjj1(:, iLayer).*exp(-i*(k(:, iLayer) - k(:, iLayer+1))*sig(iLayer+1));
    end
end

R = zeros(size(k));
T = R;
X = R;
% Boundary conditions
X(:, n) = 0; % since R_N = 0
T(:, 1) = 1;

% Solve X(:,1)
for iLayer=(n-1):-1:1
    X(:,iLayer) = exp(2*i*k(:, iLayer)*z(iLayer+1)).*((rjj1(:, iLayer) + X(:, iLayer+1).*exp(-2*i*k(:, iLayer+1)*z(iLayer+1)))./(1+rjj1(:, iLayer).*X(:,iLayer+1).*exp(-2*i*k(:, iLayer+1)*z(iLayer+1))));
end
R(:,1) = X(:,1)*1;

% Calculate for T and R for all layers.
for iLayer = 1:(n-1)
    T(:, iLayer+1) = k(:,iLayer)./k(:, iLayer+1).*(pjj1(:,iLayer).*exp(-i*(k(:, iLayer+1)-k(:, iLayer)).*z(iLayer+1)).*T(:, iLayer) ...
        -mjj1(:, iLayer).*exp(-i*(k(:, iLayer+1)+k(:, iLayer)).*z(iLayer+1)).*R(:, iLayer));
    R(:, iLayer+1) = k(:,iLayer)./k(:, iLayer+1).*(-mjj1(:,iLayer).*exp(i*(k(:, iLayer+1)+k(:, iLayer)).*z(iLayer+1)).*T(:, iLayer) ...
        +pjj1(:, iLayer).*exp(i*(k(:, iLayer+1)-k(:, iLayer)).*z(iLayer+1)).*R(:, iLayer));
end