/*=================================================================
 * Wk_U.c 
 * Calculating nonzero distances of a given coordinate (x1, y1, z1;...;xn, yn, zn)
 * 
 * MEX-file.  NOTE: MATLAB uses 1 based indexing, C uses 0 based indexing.
 *
 * Takes a N-dimensional array of doubles and returns the indices for
 * the non-zero elements in the array. Findnz works differently than
 * the FIND command in MATLAB in that it returns all the indices in
 * one output variable, where the column element contains the index
 * for that dimension.
 *
 * This is a MEX-file for MATLAB.  
 * 2007. 12. 6.
 * Copyright 1984-2006 Byeongdu Lee.
 *============================================================*/

/* $Revision: 1.1 $ */

#include "mex.h"
#include <math.h>
double Ux(double x, double w)
{
    double ret;
    if (fabs(x) <= w/2.0) ret = 1.0/w;
    else ret = 0.0;
    return ret;
}


double Wk_U(unsigned long k, double delta, double w, unsigned long numdl, double *dl, double *ul)
{
    double ret;
    double rev;
    unsigned long i;
    ret = 1.0/2.0;
    rev = 0.0;
    for (i=0;i<numdl;i++) {
        rev = rev + Ux(k*delta-dl[i], w)*ul[i]/dl[i];
    }
    return ret*rev*k*delta*delta;
}

double Iq(double qv, unsigned long S, double delta, double w, unsigned long numdl, double *dl, double *ul)
{
    double wk;
    double sink;
    double ret;
    unsigned long k;
    ret = 0.0;
    wk = 0.0;
    sink = 0.0;
    for (k=0;k<S;k++) {
        wk = Wk_U(k+1, delta, w, numdl, dl, ul);
        sink = sinc(qv*k*2*M_PI*delta);
        ret = ret + wk*sink;
    }
    ret = ret*Cq_U(qv, w);
    return ret;
}

double Cq_U(double q, double w)
{
    double x;
    if (q*w == 0.0) x = 1.0;
    else x = (M_PI*q*w)/sin(M_PI*q*w);
    return x;
}

void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */ 
    mwSize M;
    double *pind;
    double *q, *dl, *ul;
    unsigned long S;
    double delta, w;
    unsigned long numdl;
    int i;
    double Idata;
    
    /* Check for proper number of input and output arguments */    
    if (nrhs != 6) {
	mexErrMsgTxt("Six input arguments required. q, S, delta, w, dl, ul");
    } 
    if (nlhs > 1){
	mexErrMsgTxt("Too many output arguments.");
    }
    
    /* Check data type of input argument */
    if (!(mxIsDouble(prhs[0]))) {
      mexErrMsgTxt("Input array must be of type double.");
    }
    
    /* Get the number of elements in the input argument */
    // input: q, S, dl, ul, delta, w
    // output : q
//    elements=mxGetNumberOfElements(prhs[0]);
   
    /* Get the data */
    q = mxGetPr(prhs[0]);
    M = mxGetNumberOfElements(q);
    S = mxGetScalar(prhs[1]);
    delta = mxGetScalar(prhs[2]);
    w = mxGetScalar(prhs[3]);
    dl = mxGetPr(prhs[4]);
    ul = mxGetPr(prhs[5]);
    numdl = mxGetNumberOfElements(prhs[4]);
    
    /* Get the number of dimensions in the input argument. Allocate the
       space for the return argument */
    plhs[0]=mxCreateDoubleMatrix(M,1,mxREAL);
    pind=mxGetPr(plhs[0]);
	
    /* Fill in the indices to return to MATLAB. This loops through the
     * elements and checks for non-zero values. If it finds a non-zero
     * value, it then calculates the corresponding MATLAB indice and
     * assigns them into the output array.  The 1 is added to the
     * calculated indice because MATLAB is 1 based and C is zero
     * based. */
    for(i=0;i<M;i++) {
//        printf("i = %i, S = %i, w = %f, numdl = %i, q = %f\n", i, S, w, numdl, q[i]);
        Idata = Iq(q[i], S, delta, w, numdl, dl, ul);
        pind[i] = Idata;
    }
}
