/*=================================================================
 * distNzero.c 
 * Calculating nonzero distances of a given coordinate (x1, y1, z1;...;xn, yn, zn)
 * returns [Distance_between_ij, i, j]
 *
 * MEX-file.  NOTE: MATLAB uses 1 based indexing, C uses 0 based indexing.
 *
 * Takes a N-dimensional array of doubles and returns the indices for
 * the non-zero elements in the array. Findnz works differently than
 * the FIND command in MATLAB in that it returns all the indices in
 * one output variable, where the column element contains the index
 * for that dimension.
 *
 * This is a MEX-file for MATLAB.  
 * 2007. 12. 6.
 * Copyright 1984-2006 Byeongdu Lee.
 *============================================================*/

/* $Revision: 1.1 $ */

#include "mex.h"
#include <math.h>
double distance(double x1, double y1, double z1, double x2, double y2, double z2)
{
    double d;
    d = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2));
    return d;
}

void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */ 
    mwSize i,j;
    mwSize M;
    mwSize N;
    double *pr, *pind, *atm, *ff, *idi, *idj;
    double atmatm;
    int cnt;
    cnt = 0;
    
    /* Check for proper number of input and output arguments */    
    if (nrhs > 2) {
	mexErrMsgTxt("less than three input arguments required.");
    } 
    if (nlhs > 4){
	mexErrMsgTxt("Too many output arguments.");
    }
    
    /* Check data type of input argument */
    if (!(mxIsDouble(prhs[0]))) {
      mexErrMsgTxt("Input array must be of type double.");
    }
    
    /* Get the number of elements in the input argument */
	/*    elements=mxGetNumberOfElements(prhs[0]); */
    M = mxGetM(prhs[0]);
    N = mxGetN(prhs[0]);
    if (N != 3) {
        mexErrMsgTxt("Number of column is not 3.");
    }
   
    /* Get the data */
    if (nrhs >= 1) {
        pr=(double *)mxGetPr(prhs[0]);
    }
    if (nrhs == 2) {
        atm=(double *)mxGetPr(prhs[1]);
    }

    /* Get the number of dimensions in the input argument. Allocate the
       space for the return argument */
    plhs[0]=mxCreateDoubleMatrix(M*(M-1)/2,1,mxREAL);
    pind=mxGetPr(plhs[0]);
    if (nlhs > 1) {
	plhs[1]=mxCreateDoubleMatrix(M*(M-1)/2,1,mxREAL);
    ff=mxGetPr(plhs[1]);
	plhs[2]=mxCreateDoubleMatrix(M*(M-1)/2,1,mxREAL);
    idi=mxGetPr(plhs[2]);
	plhs[3]=mxCreateDoubleMatrix(M*(M-1)/2,1,mxREAL);
    idj=mxGetPr(plhs[3]);
    }

    /* Fill in the indices to return to MATLAB. This loops through the
     * elements and checks for non-zero values. If it finds a non-zero
     * value, it then calculates the corresponding MATLAB indice and
     * assigns them into the output array.  The 1 is added to the
     * calculated indice because MATLAB is 1 based and C is zero
     * based. */
    for(i=0;i<(M-1);i++) {
        for (j=i+1;j<M;j++) {
            if (nrhs == 2) {
                atmatm = atm[i]*atm[j];
            } else {
                atmatm = 1.0;
            }

            pind[cnt] = distance(pr[i], pr[i+M], pr[i+2*M],pr[j], pr[j+M], pr[j+2*M]);
            idi[cnt] = i+1;
            idj[cnt] = j+1;
            if (nlhs > 1) {
                ff[cnt] = atmatm;
            }
            cnt++;
        };
    };
}