function [Xpoint,Ypoint] = crosshair(action);
%  CROSSHAIR:  A gui interface for reading (x,y) values from a plot.
%
%  A set of mouse driven crosshairs is placed on the current axes,
%  and displays the current (x,y) values of the line plot.  There is an
%  option to provide data specific values or interpolated values.  The
%  resolution of the data specific values depends on both the data
%  resolution and the GUI interface (mainly mouse movement resolution).
%  The interpolated values appear to provide a more continuous function,
%  however they too depend on the GUI interface resolution.  There are 
%  no options for extrapolation.
%  
%  For multiple traces, plots with the same length(xdata) are
%  tracked. Each mouse click returns Xpoint,Ypoint values and selecting 
%  'done' will remove the GUI and restore the mouse buttons to previous 
%  values.  Selecting 'exit' will remove the GUI and close the figure.
%
%  Some further help is given in the tool tips of the GUI.
%
%  Useage:  x = [1:10]; y(1,:) = sin(x); y(2,:) = cos(x); x2 = x.^2;
%           figure; plot(x2,y); crosshair
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
%  Licence:        GNU GPL, no express or implied warranties
%  History: 03/96, Richard G. Cobb <cobbr@plk.af.mil>
%           08/01, Darren.Weber@flinders.edu.au
%                  replaced obsolete 'table1' with 'interp1'; fixed bug
%                  with number of 'traces'; rationalized calculations into
%                  a common subfunction for x,y point calc in 'down','up', 
%                  & 'move' button functions; added option to turn on/off
%                  interpolation and the exit button; simplified updates 
%                  to graphics using global GUI handle structure.
%           11/01, Darren.Weber@flinders.edu.au
%                  added tooltips for several GUI handles
%                  added multiple interpolation methods
%                  added GUI for data matrix indices (given no interpolation)
%                  added option to select trace nearest to mouse click point
%                  reversed order of lines in data matrix to be consistent
%                    with the value returned from the nearest trace subfunction
%                  create crosshair lines after finding all plot lines to
%                    avoid confusing them with the plot lines
%           01/02, Darren.Weber@flinders.edu.au
%                  should now work across multiple plot figures, given
%                    that all gui handles and data are now stored in the
%                    plot figure 'userdata' handle.
%                  added functionality to move smoothly from interpolation
%                    back to the delimited data via the "next/previous" 
%                    buttons.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~exist('action','var') 
    action = 'init';
elseif isempty(action)
    action = 'init';
end

XHR_HANDLES = get(gcbf,'userdata');

if strcmp(action, 'init');
    
    % Paint GUI
    XHR_HANDLES = INIT;
    
    % Update and return values
    XHR_HANDLES = updateDATA(XHR_HANDLES);
    Xpoint = get(XHR_HANDLES.xvalue,'Value');
    Ypoint = get(XHR_HANDLES.yvalue,'Value');
    
% Mouse Click Down
elseif strcmp(action,'down');
    
    set(XHR_HANDLES.plot,'WindowButtonMotionFcn','crosshair(''move'');');
    set(XHR_HANDLES.plot,'WindowButtonUpFcn','[Xpoint,Ypoint] = crosshair(''up'');');
    
    XHR_HANDLES = updateDATA(XHR_HANDLES);
    Xpoint = get(XHR_HANDLES.xvalue,'Value');
    Ypoint = get(XHR_HANDLES.yvalue,'Value');
    
% Mouse Drag Motion
elseif strcmp(action,'move');
    
    XHR_HANDLES = updateDATA(XHR_HANDLES);
    Xpoint = get(XHR_HANDLES.xvalue,'Value');
    Ypoint = get(XHR_HANDLES.yvalue,'Value');
    
% Mouse Click Up
elseif strcmp(action,'up');
    
    set(XHR_HANDLES.plot,'WindowButtonMotionFcn',' ');
    set(XHR_HANDLES.plot,'WindowButtonUpFcn',' ');
    
    XHR_HANDLES = updateDATA(XHR_HANDLES);
    Xpoint = get(XHR_HANDLES.xvalue,'Value');
    Ypoint = get(XHR_HANDLES.yvalue,'Value');
    
% Next or Previous X point
elseif or(strcmp(action,'nextx'),strcmp(action,'prevx'));
    
    XHR_HANDLES = moveX(XHR_HANDLES,action);
    Xpoint = get(XHR_HANDLES.xvalue,'Value');
    Ypoint = get(XHR_HANDLES.yvalue,'Value');

% Exit crosshairs GUI
elseif or(strcmp(action,'done'),strcmp(action,'exit'));
    
    XHR_HANDLES = get(gcbf,'userdata');
    
    Xpoint = get(XHR_HANDLES.xvalue,'Value');
    Ypoint = get(XHR_HANDLES.yvalue,'Value');
    
    delete(XHR_HANDLES.interp);
    delete(XHR_HANDLES.xline);
    delete(XHR_HANDLES.yline);
    delete(XHR_HANDLES.xvalueLabel);
    delete(XHR_HANDLES.xvalue);
    delete(XHR_HANDLES.xindexLabel);
    delete(XHR_HANDLES.xindex);
    delete(XHR_HANDLES.xnext);
    delete(XHR_HANDLES.xprev);
    delete(XHR_HANDLES.yvalueLabel);
    delete(XHR_HANDLES.yvalue);
    delete(XHR_HANDLES.yindexLabel);
    delete(XHR_HANDLES.yindex);
    delete(XHR_HANDLES.trace);
    delete(XHR_HANDLES.traceLabel);
    delete(XHR_HANDLES.traceNearest);
    delete(XHR_HANDLES.done);
    delete(XHR_HANDLES.exit);
    
    if strcmp(action,'exit');
        
        close(XHR_HANDLES.plot);
    else
        
        set(XHR_HANDLES.plot,'WindowButtonUpFcn','');
        set(XHR_HANDLES.plot,'WindowButtonMotionFcn','');
        set(XHR_HANDLES.plot,'WindowButtonDownFcn',XHR_HANDLES.button);
        
        refresh(XHR_HANDLES.plot);
    end
    
    clear XHR_HANDLES;
    return;
    
end

set(gcf,'userdata',XHR_HANDLES);

return;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [H] = moveX(H,move)
    
    interp = get(H.interp,'Value');
    if (interp > 1)
        xdata = H.DATA.xdata(:,H.DATA.yindex);
        [H.DATA.xindex] = NearestXYArrayPoint( xdata, H.DATA.xpoint, move );
    end
    
    if strcmp(move,'nextx')
        % Increase current xindex by one
        if(interp == 1), H.DATA.xindex = H.DATA.xindex + 1; end
        
    elseif strcmp(move,'prevx')
        % Decrease current xindex by one
        if(interp == 1), H.DATA.xindex = H.DATA.xindex - 1; end
        
    end
    
    % Get x point/value at new xindex
    s = size(H.DATA.xdata);
    if  H.DATA.xindex <= 0
        H.DATA.xindex = 1;
    elseif H.DATA.xindex >= s(1)
        H.DATA.xindex = s(1);
    end
    H.DATA.xpoint = H.DATA.xdata(H.DATA.xindex,H.DATA.yindex);
    H.DATA.ypoint = H.DATA.ydata(H.DATA.xindex,H.DATA.yindex);
    
    set(H.interp,'Value',1);
    updateGUI(H);
    
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ H ] = updateDATA( H )

    CurrentPoint  = get(H.axis,'Currentpoint');
    H.DATA.xpoint = CurrentPoint(1,1);
    H.DATA.ypoint = CurrentPoint(1,2);
    
    doNearTrace   = get(H.traceNearest,'Value');
    
    if (doNearTrace > 0)
        
        % Get new yindex for nearest trace
        [ H.DATA.xpoint, ...
          H.DATA.xindex, ...
          H.DATA.ypoint, ...
          H.DATA.yindex ] = NearestXYMatrixPoint( H.DATA.xdata,...
                                                  H.DATA.ydata,...
                                                  H.DATA.xpoint,...
                                                  H.DATA.ypoint);
    else
        H.DATA.yindex = get(H.trace,'Value');
    end
    
    CurrentPoint  = get(H.axis,'Currentpoint');
    H.DATA.xpoint = CurrentPoint(1,1);
    H.DATA.ypoint = CurrentPoint(1,2);
    
    H = interpY(H);
    H = updateGUI(H);

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ H ] = updateGUI( H )

    InterpMethod = get(H.interp,'Value');
    if (InterpMethod > 1)
        % There is no specific matrix x-index for 
        % an interpolated point, but the nearest xindex
        % is always returned from the interp function below
        % so that next/prev move function works correctly
        set(H.xindex,'String','');
    else
        set(H.xindex,'String',num2str(H.DATA.xindex));
    end
    set(H.xindex,'Value',H.DATA.xindex);
    
    tracestr = sprintf('%d',H.DATA.yindex);
    set(H.yindex,'String',tracestr,'Value',uint16(H.DATA.yindex));
    set(H.trace,'Value',uint16(H.DATA.yindex));
    
    % Create the crosshair lines on the figure, crossing at the x,y point
    x_rng  = get(H.axis,'Xlim');
    y_rng  = get(H.axis,'Ylim');
    set(H.xline,'Xdata',[H.DATA.xpoint H.DATA.xpoint],'Ydata',y_rng);
    set(H.yline,'Ydata',[H.DATA.ypoint H.DATA.ypoint],'Xdata',x_rng);
    
    % Update the x,y values displayed for the x,y point
    xstring = sprintf('%14.6f',H.DATA.xpoint);
    ystring = sprintf('%14.6f',H.DATA.ypoint);
    set(H.xvalue,'String',xstring,'Value',H.DATA.xpoint);
    set(H.yvalue,'String',ystring,'Value',H.DATA.ypoint);

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ H ] = interpY( H )
    
    % In this function, xdata & ydata are arrays, not matrices
    xdata = H.DATA.xdata(:,H.DATA.yindex);
    ydata = H.DATA.ydata(:,H.DATA.yindex);
    
    if      H.DATA.xpoint >= max(xdata)                
            H.DATA.xpoint  = max(xdata);
            H.DATA.xindex  = find(xdata == max(xdata));
            H.DATA.ypoint  = ydata(H.DATA.xindex);
            return;
    elseif  H.DATA.xpoint <= min(xdata)
            H.DATA.xpoint  = min(xdata);
            H.DATA.xindex  = find(xdata == min(xdata));
            H.DATA.ypoint  = ydata(H.DATA.xindex);
            return;
    end
    
    % 'none|nearest|linear|spline|cubic'
    interp = get(H.interp,'Value');
    
    switch interp
    case 1
        % Given that xdata & ydata are the same length arrays,
        % we can find the ypoint given the nearest xpoint.
        [H.DATA.xindex, H.DATA.xpoint] = NearestXYArrayPoint( xdata, H.DATA.xpoint );
        H.DATA.ypoint = ydata(H.DATA.xindex);
    case 2
        H.DATA.ypoint = interp1( xdata, ydata, H.DATA.xpoint, 'nearest' );
    case 3
        H.DATA.ypoint = interp1( xdata, ydata, H.DATA.xpoint, 'linear' );
    case 4
        H.DATA.ypoint = interp1( xdata, ydata, H.DATA.xpoint, 'spline' );
    case 5
        H.DATA.ypoint = interp1( xdata, ydata, H.DATA.xpoint, 'cubic' );
    otherwise
        %use default (linear in matlabR12)
        H.DATA.ypoint = interp1( xdata, ydata, H.DATA.xpoint );
    end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ index, point ] = NearestXYArrayPoint( data_array, point, type )
    
    if ~exist('type','var') type = ''; end

    % In this function, input data_array is an array, not a matrix.
    % This function returns the data point in the array
    % that has the closest value to the value given (point).  In
    % the context of 'crosshair' the point is a mouse position.
    
    if      point >= max(data_array)
            point  = max(data_array);
            index  = find(data_array == point);
            return;
    elseif  point <= min(data_array)
            point  = min(data_array);
            index  = find(data_array == point);
            return;
    end
    
    data_sorted = sort(data_array);
    
    greater = find(data_sorted > point);
    greater_index = greater(1);
    
    lesser = find(data_sorted < point);
    lesser_index = lesser(length(lesser));
    
    greater_dif = data_sorted(greater_index) - point;
    lesser_dif  = point - data_sorted(lesser_index);
    
    if strcmp(type,'nextx'),
        index = greater_index;
    elseif strcmp(type,'prevx'),
        index = lesser_index;
    else
        if (greater_dif < lesser_dif)
            index = find(data_array == data_sorted(greater_index));
        else
            index = find(data_array == data_sorted(lesser_index));
        end
    end
    point = data_array(index);
    
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ xpoint, xindex, ypoint, yindex ] = NearestXYMatrixPoint( Xdata, Ydata, xpoint, ypoint )

    % In this function, Xdata & Ydata are matrices of the same dimensions.
    % This function attempts to find the nearest values in Xdata & Ydata
    % to the mouse position (xpoint, ypoint).
    
    % It is assumed that Xdata has identical columns, so we only really
    % need the first column to find the nearest value to xpoint.
    
    [ xindex, xpoint ] = NearestXYArrayPoint( Xdata(:,1), xpoint );
    
    % Now, given the xpoint, we can select just that row of the
    % Ydata matrix corresponding to the xpoint.
    ydata = Ydata(xindex,:);
    
    % The ydata array is searched in same manner as the xdata
    % array for the nearest value.
    [ yindex, ypoint ] = NearestXYArrayPoint( ydata, ypoint );
    
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [H] = INIT

    H.plot = gcf; % Get current figure handles
    H.axis = gca; % Get current axis handles
    
    % store current button fcn
    H.button = get(H.plot,'WindowButtonDownFcn');
    % set XHR button down fcn
    set(H.plot,'WindowButtonDownFcn','crosshair(''down'');');
    
    interpstr     =   'none|nearest|linear|spline|cubic';
    H.interp      =   uicontrol('Style','popup','Units','Normalized',...
                              'Position',[.00 .95 .10 .05],...
                              'Tag','INTERP',...
                              'TooltipString','MATLAB INTERPOLATION METHODS (none = raw values).', ...
                              'String',interpstr);
                          
    H.xvalueLabel = uicontrol('Style','edit','Units','Normalized',...
                              'Position',[.10 .95 .10 .05],...
                              'Tag','XVALUELAB',...
                              'BackGroundColor',[.7 .7 .7],'String','X value');
    H.xvalue      = uicontrol('Style','edit','Units','Normalized',...
                              'Position',[.20 .95 .15 .05],...
                              'Tag','XVALUE',...
                              'BackGroundColor',[ 0 .7 .7],'String',' ');                      
    H.yvalueLabel = uicontrol('Style','edit','Units','Normalized',...
                              'Position',[.35 .95 .10 .05],...
                              'Tag','YVALUELAB',...
                              'BackGroundColor',[.7 .7 .7],'String','Y value');
    H.yvalue      = uicontrol('Style','edit','Units','Normalized',...
                              'Position',[.45 .95 .15 .05],...
                              'Tag','YVALUE',...
                              'BackGroundColor',[ 0 .7 .7],'String',' ');
    
    H.yindexLabel = uicontrol('Style','edit','Units','Normalized',...
                              'Position',[.92 .85 .08 .05],...
                              'Tag','YINDEXLAB',...
                              'BackGroundColor',[.7 .7 .7],'String','Yindex',...
                              'TooltipString','Y index into plot Y data matrix.  Same as trace number.');
    H.yindex      = uicontrol('Style','edit','Units','Normalized',...
                              'Position',[.92 .80 .08 .05],...
                              'Tag','YINDEX',...
                              'BackGroundColor',[ 0 .7 .7],'String','');
    H.xindexLabel = uicontrol('Style','edit','Units','Normalized',...
                              'Position',[.92 .75 .08 .05],...
                              'Tag','XINDEXLAB',...
                              'BackGroundColor',[.7 .7 .7],'String','Xindex',...
                              'TooltipString','X index into plot Y data matrix.  Only available for interpolation = ''none''.');
    H.xindex      = uicontrol('Style','edit','Units','Normalized',...
                              'Position',[.92 .70 .08 .05],...
                              'Tag','XINDEX',...
                              'BackGroundColor',[ 0 .7 .7],'String','');
    H.xprev       = uicontrol('Style','Push','Units','Normalized',...
                              'Position',[.92 .65 .08 .05],...
                              'String','Prev X',...
                              'Tag','XPREV',...
                              'TooltipString','Goto Previous X Index (no interpolation).',...
                              'CallBack','crosshair(''prevx'');');
    H.xnext       = uicontrol('Style','Push','Units','Normalized',...
                              'Position',[.92 .60 .08 .05],...
                              'String','Next X',...
                              'Tag','XNEXT',...
                              'TooltipString','Goto Next X Index (no interpolation).',...
                              'CallBack','crosshair(''nextx'');');
                              
    H.done        = uicontrol('Style','Push','Units','Normalized',...
                              'Position',[.80 .00 .10 .05],...
                              'String','Done',...
                              'Tag','DONE',...
                              'TooltipString','Close crosshair', ...
                              'CallBack','crosshair(''done'');');
    H.exit        = uicontrol('Style','Push','Units','Normalized',...
                              'Position',[.90 .00 .10 .05],...
                              'String','Exit',...
                              'Tag','EXIT',...
                              'TooltipString','Close crosshair and Figure', ...
                              'CallBack','crosshair(''exit'');');
                          
    %%%%%%%%%%%%%%%%%%%%%%%%%
    % Get Line Data from Plot
    
    % Lines are referenced as axis children, among other
    % axis children; so first get all axis children
    sibs = get(H.axis,'Children');
    
    % Now search axis children for any line types.
    % Because the columns of the y data matrix in a plot
    % command seem to be reversed in the axis children, 
    % count down from max sibs to the first sib.
    lines = 0;
    H.DATA.xdata = [];
    H.DATA.ydata = [];
    H.DATA.xpoint = [];
    H.DATA.ypoint = [];
    H.DATA.xindex = 1;
    H.DATA.yindex = 1;
    i = max(size(sibs));
    while i >= 1
        if strcmp(get(sibs(i),'Type'),'line')
            
            % OK, found a line among the axis children.
            lines = lines + 1;
            
            % put line data into a column of DATA.xdata|DATA.ydata
            H.DATA.xdata(:,lines) = get(sibs(i),'XData').';
            H.DATA.ydata(:,lines) = get(sibs(i),'YData').';
        end
        i = i - 1;
    end
    
    % 'traces' string variable must be in ascending order
    traces  = '';
    i = 1;
    while i <= lines;
        if i < lines
            tracelabel = sprintf('Column %4d|',i);            
        else
            tracelabel = sprintf('Column %4d',i);
        end
        traces = strcat(traces,tracelabel);
        i = i + 1;
    end
    
    % If more than one line, provide GUI for line selection
    
    % Switch off||on Trace Selection GUI
    Vis = 'Off';
    if lines > 1,
        Vis = 'On';
    elseif lines == 0
        error('No lines found in the current plot window\n');
    end
    
    % Create Trace Index GUI
    H.traceLabel   = uicontrol('Style','Edit', 'Units','Normalized',...
                               'Position',[.00 .00 .15 .05],...
                               'Tag','TRACELABEL',...
                               'Visible',Vis,'String','Select Trace :',...
                               'TooltipString','Select trace to follow with crosshairs.');
    H.trace        = uicontrol('Style','Popup','Units','Normalized',...
                               'Position',[.15 .00 .20 .05],...
                               'Tag','TRACESWITCH',...
                               'BackGroundColor','w','String',traces,...
                               'Visible',Vis,...
                               'CallBack',['[Xpoint,Ypoint] = crosshair(''up'');',]);    
    H.traceNearest = uicontrol('Style','checkbox', 'Units','Normalized',...
                               'Position',[.36 .00 .19 .05],...
                               'Tag','TRACENEAREST',...
                               'Visible',Vis,'String','Nearest Trace','Value',0,...
                               'TooltipString','Trace nearest to mouse click. Updates ''Select Trace''; switch off to keep trace constant.');
    
    % Set X,Y cross hair lines
    % Do this after finding all the line axis children
    % to avoid confusing these lines with those of the
    % plot itself (counted above).
    x_rng = get(H.axis,'Xlim');
    y_rng = get(H.axis,'Ylim');
    H.xline = line(x_rng,[y_rng(1) y_rng(1)]);
    H.yline = line(x_rng,[y_rng(1) y_rng(1)]);
    set(H.xline,'Color','r','EraseMode','xor','Tag','XLINE');
    set(H.yline,'Color','r','EraseMode','xor','Tag','YLINE');
    
return
