%DIFFRACTION calculates diffraction pattern of monochromatic
%   light through a transmission grating for arbitrary slit sizes 
%   and slit transmission coefficients.
%   This MATLAB program is intended as a pedagogical example
%   of how diffraction and interference arise strictly from
%   the wave nature of light.
%   invoke diffraction(CELLS,SLITSIZE1,SLITSIZE2,T1,T2)
%   where CELLS is the number of pairs of slits, SLITSIZE1 is
%   the width of slit 1 in meters, SLITSIZE2 is the width of slit 2 in meters, 
%   T1 is the transmission coefficient of slit 1, and T2 is the 
%   transmission coefficient of slit 2. (0<(T1,T2)<1)
%   try diffraction(1,1e-5,1e-5,1,0) for a simple example of 1 slit diffraction, 
%   or diffraction(2,1e-5,1e-5,1,0) for Young's two-slit experiment.
%
%   PLEASE READ THE CODE FOR MORE DETAILS
%
%   Ian Appelbaum MIT Dec 1999
%   304 McKay Labs
%   Harvard University
%   Cambridge MA 02138
function mag=diffraction(CELLS,SLITSIZE1,SLITSIZE2,T1,T2)
tic
DISTANCE=5; %distance from slit to screen in meters
WAVELENGTH=633e-9; %wavelength of light in meters (633 nm is HeNe laser line)
K=2*pi/WAVELENGTH; %wavenumber
CELLSIZE=SLITSIZE1+SLITSIZE2;

%the following constants are calculated from the inputs assuming:
% SLITX >> WAVELENGTH 
SLITRES=WAVELENGTH/100; %resolution of position of sources at slit
%SCREENRES < WAVELENGTH/SLITX
SCREENRES=DISTANCE/(CELLS*10)*WAVELENGTH/mean([SLITSIZE1 SLITSIZE2]); %resolution of pattern at screen
%SCREENLENGTH > WAVELENGTH/SLITX
SCREENLENGTH=3*DISTANCE*WAVELENGTH/mean([SLITSIZE1 SLITSIZE2]); %distance from center point to end of screen in meters

mag=zeros(0,2);
for screenpt=0:SCREENRES:SCREENLENGTH
   wavesum=0;
   for cellnum=0:(CELLS-1)
      for sourcept=0:SLITRES:SLITSIZE1 %slit2
         horizpos=(screenpt-(cellnum*CELLSIZE+sourcept));
         x=abs(horizpos+i*DISTANCE); %this is an efficient way to use pythagorean thm.
         wavesum=wavesum+T1*exp(i*K*x); %add up the wave contribution from the first slit
      end
      for sourcept=0:SLITRES:SLITSIZE2 %slit2
         horizpos=(screenpt-(cellnum*CELLSIZE+SLITSIZE1+sourcept));
         x=abs(horizpos+i*DISTANCE);%this is an efficient way to use pythagorean thm.
         wavesum=wavesum+T2*exp(i*K*x);%add up the wave contribution from the second slit
      end
   end
   intensity=(abs(wavesum))^2; %intensity is electric field squared
   newdata=[screenpt*100 intensity/(CELLS*CELLSIZE/SLITRES)]; %normalize intensity so it is ~ 1
   disp(newdata) %this is just so user knows progress
   mag=cat(1,mag,newdata); %put data in array mag for plotting
end
disp(toc)
plot( mag(:,1),mag(:,2)/max(mag(:,2)) );
title([num2str(CELLS) ' cells: SLIT1=' num2str(SLITSIZE1) ' SLIT2=' ...
      num2str(SLITSIZE2) ' T1=' num2str(T1) ' T2=' num2str(T2) ]);
xlabel('position at screen (centimeters)')
ylabel('light intensity (arb)')