function varargout = efield(varargin)
% EFIELD Spatial distribution of x-ray excited complex electric field.
%
%   EF = EFIELD(ALPHA,PROFILE,WAVELENGTH) calculates spatial distribution 
%       of x-ray excited electric field (normalized to the incident beam)
%       in films by Parratt's recursion.
%
%   [EF,R,T,KZ] = EFIELD(ALPHA,PROFILE,WAVELENGTH) also returns R and T, 
%       which are the electric field amplitudes of reflected and transmitted
%       waves within  the film.
%
%   EF = EFIELD(ALPHA,PROFILE,WAVELENGTH,PLOTFLAG) also plots electric
%       field intensity (EFI) distribution map.
%
%   Input Argument:
%       ALPHA       : 1 x M list. Incident/exit angle in the reference
%               material. Unit: degree.
%       PROFILE     : N+1 x 3 matrix. Film depth, dispersion (delta) and
%               absorption (beta) profile. N is the total number of
%               interfaces. 1st column: depth (Unit: A) of each interface
%               with z-axis pointing down into the film. 2nd and 3rd 
%               columns: dispersion (delta) and absorption (beta) 
%               coefficients in the slab above the correponding interface 
%               defined in the 1st column. NOTE: PROFILE(1,2:3) are delta 
%               and beta of the reference material above interface 
%               PROFILE(1,1); PROFILE(N+1,2:3) are used as delta and beta 
%               of the substrate; PROFILE(N+1,1) can be any arbitrary 
%               number and is not used in the calculation.
%       WAVELENGTH  : Wavelength of x-ray in the vacuum. Unit: A.
%       PLOTFLAG    : Flag to plot: 'plot'.
%
%   Output Argument:
%       EF:     N+1 x M. Complex electric field.
%       R:      N+1 x M. Complex electric field amplitude of reflected waves.
%       T:      N+1 x M. Complex electric field amplitude of transmitted waves.
%       KZ:     N+1 x M. Complex kz within the film.
%
%   Reference:
%   1) Tolan, M. (1999) X-Ray Scattering from Soft-Matter Thin Films, Springer
%   2) Parratt, L.G. (1954). Phys. Rev. 95, 359-369
%   3) Als-Nielsen, J. (2001) Elements of Modern X-ray Physics, Wiley John & Sons
%
%   Copyright 2005-2006 Zhang Jiang
%   $Revision: 1.0 $  $Date: 2005/05/03$
%   $Revision: 1.1 $  $Date: 2005/06/25$ : Add options to output electric
%       field amplitudes of reflected and transmitted waves, and kz within the
%       film as a function of depth and angle.
%
% See also REFCONV, PARRATT, EDPROFILE, REFRAC

if nargin ~=3 && nargin ~=4
    error('Invalid input argument.');
    return;
end
alpha   = varargin{1};              % alpha in the first medium (M points)
profile = varargin{2};              % profile of sample; N+1 points with 
                                    % 1st the media above sample and last 
                                    % point the substrate
wavelength = varargin{3};           % wavelength of beam in vacuum
plotflag = 0;   
if nargin == 4 
    if strcmp(lower(varargin{4}),'plot')
        plotflag = 1;
    else
        error('Invalid input argumnet.');
        return;
    end
end

nLayer = size(profile,1);           % N+1 layers including 'vacuum' and substrate
z = profile(:,1);                  % slice position; the last z(N+1) dos not
                                    % count, simply stands for substrate
delta = profile(:,2);
beta = profile(:,3);
k = 2*pi/wavelength;                % wave vector in vacuum

% --- construct refractive index (N+1 x 1)
n = 1-delta+i*beta;

% --- kz in the reference material
kz = n(1)*k*sin(alpha*pi/180);                          
if size(kz,2) == 1      % transfer kz (1 x M)
    kz = kz';
end

% --- get kz in vacuum (N+1 x 1)
kzInVac = sqrt(kz.^2+k^2-k^2*n(1)^2);

% --- kz in each layer (N+1 x M)
KzLayer = sqrt( (n*ones(1,size(kzInVac,2))).^2*k^2 - k^2 + (ones(size(n,1),1)*kzInVac).^2 );

% --- Reflectance coefficients (no multiple scattering) (N x M)
r = ( KzLayer(1:end-1,:) - KzLayer(2:end,:) ) ./ ( KzLayer(1:end-1,:) + KzLayer(2:end,:) );

% --- Reflectance from more layers (with multiple scattering) (N x M)
X = zeros(nLayer,length(kz));
X(end,:) = 0;
for iInterface = nLayer-1:-1:1          % N-1 reflectances needed to be calculated
        X(iInterface,:) = (r(iInterface,:)+X(iInterface+1,:).*exp(-2*i*KzLayer(iInterface+1,:)*z(iInterface)))  ./...
            (1+r(iInterface,:).*X(iInterface+1,:).*exp(-2*i*KzLayer(iInterface+1,:)*z(iInterface)))  .*...
            exp(2*i*KzLayer(iInterface,:)*z(iInterface));
end

% --- calcuate R (N+1 x M) and T (N+1 x M) in each layer
R = zeros(nLayer,length(kz));
T = zeros(nLayer,length(kz));
R(1,:)      = X(1,:);
T(1,:)      = 1;
for iInterface = 1:nLayer-1
    R(iInterface+1,:) = ( T(iInterface,:).*(-r(iInterface,:)).*exp(i*(KzLayer(iInterface+1,:)+KzLayer(iInterface,:))*z(iInterface)) + ...
                                              R(iInterface,:).*exp(i*(KzLayer(iInterface+1,:)-KzLayer(iInterface,:))*z(iInterface)) ) ./...
                          ( 1-r(iInterface,:) );
    T(iInterface+1,:) = ( R(iInterface,:).*(-r(iInterface,:)).*exp(-i*(KzLayer(iInterface+1,:)+KzLayer(iInterface,:)).*z(iInterface)) + ...
                                              T(iInterface,:).*exp(-i*(KzLayer(iInterface+1,:)-KzLayer(iInterface,:)).*z(iInterface)) ) ./...
                          ( 1-r(iInterface,:) );
end
R(end,:)    = X(end,:);

% --- calculate complex electric field
ef = T.*exp(i*z*ones(1,size(KzLayer,2)).*KzLayer) + R.*exp(-i*z*ones(1,size(KzLayer,2)).*KzLayer);

if plotflag == 1
    % plot EFI map
    imagesc(alpha,profile(:,1),abs(ef).^2);
    set(gca,'YDir','normal');
    xlabel('Angle (deg.)');
    ylabel('Depth (A)');
    figure(gcf);
%     % plot phase angle map
%     figure;
%     imagesc(alpha,profile(:,1),unwrap(angle(ef)*180/pi));
%     set(gca,'YDir','normal');
%     xlabel('Incident Angle (deg.)');
%     ylabel('Depth (A)');
%     title('Electric Field Phase');
%     colorbar;
end
if nargout == 1
    varargout{1} = ef;
elseif nargout == 4
    varargout{1} = ef;
    varargout{2} = R;
    varargout{3} = T;
    varargout{4} = KzLayer;
else
    error('Invalid output argument');
end
return;