function s = sofq(qz,qr,B,qu,sigma0)
% SOFQ Structor factor for a isotropic rough surface.
%   S = SOFQ(QZ,QR,B,QU,SIGMA0) Returns the structure factor of a isotropic
%       rough surface by capillary wave theory.
%
%   Input Argument:
%       QZ      : Vertical wave vector transder;    (m^-1)
%       QR      : Inplane wave vector transfer;     (m^-1)
%       B       : Defined as KB*T/gamma/pi;         
%       QU      : Upper cutoff (usually related to the molecular size);
%       SIGMA0  : Surface roughness.    (A)
%
%   Output Argument:
%       S       : Structor factor. (m^4)
%
%   Reference:
%       1. Laurence Lurio, et.al., Macromolecules 36, 5704-5709 (2003).
%       2. S. K. Sinha, et.al., Phys. Rev. B 38, 2297 (1988).
%
%   Copyright 2005 Zhang Jiang
%   $Revision: 1.0 $  $Date: 2005/12/16$

Euler   = 0.577215665;
eta     = B*qz.^2/2;
yout    = (2*pi./qz.^2).*exp(-sigma0.^2.*qz.^2);
yout    = yout.*(4*pi./qu/exp(Euler)).^eta;
yout    = yout.*(qr/2).^(eta/2-1);
yout    = yout.*(2*pi/qu).^(1-eta/2);
yout    = yout.*besselk((1-eta/2),2*pi.*qr/qu)./gamma(eta/2);
s       = yout;
return;