classdef BOXmodel
    % Fq or Pq computing method
    %   1. Fqcal for given qv
    %   2. Fqfft and will generate qx, qy and qz
    %   3. Pqdebye for given q -- eventually added
    % Drawing isosurface of Nelectrons boxes
    %   .drawiso
    %
    % Examples:
    % pxxx = PDB('xxx.pdb');
    % md = BOXmodel(pxxx);
    % Calculate Fq 
    %   md.qv = [qx, qy, qz];
    %   md.Fqcal
    % Calculate Fq with FFT
    %   md.Fqfft
    %   md.Pqorientavg
    % Calculate Pq with debye function
    %   md.q = 1:10;
    %   md.debyePq

    properties (SetAccess = public)
        resolution
        boxvolume
        resName;
        Atom;
        atomName
        X
        Y
        Z
        Nelectrons
        xbox
        ybox
        zbox
        cpb
        isosurf
        boxrange
        Fq
        Pq
        q
        qx % 1d array for qx for 3d Nelectrons boxes.
        qy % 1d array for qy for 3d Nelectrons boxes.
        qz % 1d array for qz for 3d Nelectrons boxes.
        Pq1d
        qv % qnum x 3 matrix for q vectors.
        pr
        r
        rhoprofile
        Fqmean
    end
    methods
        function obj = BOXmodel(varargin)
            pdb = varargin{1};
            resolution = 20;
            if numel(varargin)>1
                for i=2:2:numel(varargin)
                    eval(sprintf('%s = %f;', varargin{i}, varargin{i+1}));
                end
            end
            if isa(pdb, 'Atom')
                pdb.X = pdb.X(:);
                pdb.Y = pdb.Y(:);
                pdb.Z = pdb.Z(:);
                pdb.Atom = pdb.Atom(:);
                % pdb needs to have X, Y, Z and atom.
    %            if nargin < 2
    %                resolution = 20;
    %            end
                xmin = min(pdb.X);
                xmax = max(pdb.X);
                ymin = min(pdb.Y);
                ymax = max(pdb.Y);
                zmin = min(pdb.Z);
                zmax = max(pdb.Z);
                m = min([xmin, ymin, zmin]); m = floor(m);
                n = max([xmax, ymax, zmax]); n = ceil(n);
                x = m:resolution:n;
                obj.boxrange = x;
            elseif isfield(pdb, "vox")
                obj.xbox = pdb.X(:);
                obj.ybox = pdb.Y(:);
                obj.zbox = pdb.Z(:);
                obj.Nelectrons = pdb.vox;
            end
            obj.resolution = resolution;
            
            if isa(pdb, 'PDB')
                obj.resName = pdb.resName;
                obj.atomName = pdb.atomName;
                obj.Atom = pdb.Atom;
                obj.X = pdb.X;
                obj.Y = pdb.Y;
                obj.Z = pdb.Z;
                obj = PDB_atom2residue(obj);
            end
            obj = obj.makebox;
        end
        function obj=makebox(obj)
            resol = obj.resolution;
            boxvol = resol^3;
            obj.boxvolume = boxvol;
            if isa(obj, 'Atom')
                x = obj.boxrange;
                [a, ~,centerpositionofbox] = histcn([obj.X(:),obj.Y(:),obj.Z(:)], x, x, x, 'AccumData', obj.Atom);
                % cpb = center positions of the small resolution boxes.
                % a : atomic density
                obj.Nelectrons = a;
                obj.cpb = centerpositionofbox;
                t = isosurface(centerpositionofbox{1}, centerpositionofbox{2},centerpositionofbox{3}, a);
    
                boxx = centerpositionofbox{1};
                boxy = centerpositionofbox{2};
                boxz = centerpositionofbox{3};
                [X0,Y0,Z0] = meshgrid(boxx, boxy, boxz);
                obj.xbox = X0(:);
                obj.ybox = Y0(:);
                obj.zbox = Z0(:);
                obj.isosurf = t;
            elseif isprop(obj, 'Nelectrons')
                Xv = reshape(obj.xbox, size(obj.Nelectrons));
                Yv = reshape(obj.ybox, size(obj.Nelectrons));
                Zv = reshape(obj.zbox, size(obj.Nelectrons));
                t = isosurface(Xv, Yv, Zv, obj.Nelectrons);
                obj.isosurf = t;
            end
        end
        function obj=removeEmptyBox(obj)
            ed = obj.Nelectrons(:)';
            t = ed == 0;
            if ~sum(t)
                return;
            end
            x = obj.xbox;
            y = obj.ybox;
            z = obj.zbox;
%             boxx = obj.cpb{1};
%             boxy = obj.cpb{2};
%             boxz = obj.cpb{3};
%             [X0,Y0,Z0] = meshgrid(boxx, boxy, boxz);
%             x = X0(:)';
%             y = Y0(:)';
%             z = Z0(:)';
            x(t) = [];
            y(t) = [];
            z(t) = [];
            ed(t) = [];
            obj.Nelectrons = ed;
            obj.xbox = x;
            obj.ybox = y;
            obj.zbox = z;
        end
        function obj=Fqcal(obj)
            % qv is needed. 
            if isempty(obj.qv)
                if ~isempty(obj.qx)
                    obj = obj.makevectq;
                    fprintf('qv will be generated from qx, qy and qz\r');
                else
                    error('qv is required')
                end
            end
            if ~isempty(obj.qx)
                Fqsize = [numel(obj.qx), numel(obj.qy), numel(obj.qz)];
            else
                Fqsize = 1;
            end
            if numel(obj.qv(:,1)) ~= prod(Fqsize)
                Fqsize = size(obj.qv(:,1));
            end
            obj = obj.removeEmptyBox;
            %q = [qx0, qy0, qz0];
            %r
            x = obj.xbox;
            y = obj.ybox;
            z = obj.zbox;
            ed = obj.Nelectrons;
            
            parPos = [x(:)';y(:)';z(:)'];
%             if isprop(obj, 'sizescale')
%                 if ~isempty(obj.sizescale)
%                     if obj.sizescale > 0
%                         parPos = parPos*obj.sizescale;
%                     end
%                 end
%             end
            qvector = obj.qv;
            
            matlimit = 1E8;
            Nq = numel(obj.qv(:,1));
            fprintf('Total Number of q is %i\n', Nq);
            fprintf('Total Number of boxes is %i\n', numel(x));
            if Nq*numel(x) > matlimit
                Fqtemp = [];
                qnum = fix(matlimit/numel(x));
                if qnum < 1
                    error('qnum is less than 1');
                end
                edseg = repmat(ed, [qnum, 1]);
                qseg = ceil(Nq/qnum);
                for i=1:qseg
                    if i<qseg
                        rng = ((i-1)*qnum+1):(i*qnum);
                    else
                        rng = ((i-1)*qnum+1):Nq;
                        edseg = repmat(ed, [numel(rng), 1]);
                    end
                    fprintf('Step %i/%i\r', i, qseg) 
                    Fqtemp = [Fqtemp;sum(edseg.*exp(-sqrt(-1)*qvector(rng,:)*parPos),2)];
                end
                obj.Fq = reshape(Fqtemp, Fqsize);
            else
                ed = repmat(ed, [Nq, 1]);
                obj.Fq = sum(ed.*exp(-sqrt(-1)*qvector*parPos),2);
                obj.Fq = reshape(obj.Fq, Fqsize);
            end
            obj.Pq = abs(obj.Fq).^2;
        end
        function obj=Fqfft(obj)
            Yv = obj.Nelectrons;
            qnum = numel(obj.boxrange)*3;
            if qnum > 300
                qnum = 300;
            end
            q1d = linspace(-1/max(obj.boxrange)/2, 1/max(obj.boxrange)/2, qnum);
            q1d = q1d/obj.resolution*max(obj.boxrange)*(2*pi);
            for p = 1:length(size(obj.Nelectrons))
                Yv = fft(Yv,qnum,p);
            end
            obj.Fq = Yv;
            obj.Fq = fftshift(obj.Fq, 1);
            obj.Fq = fftshift(obj.Fq, 2);
            obj.Fq = fftshift(obj.Fq, 3);
            obj.Pq = abs(obj.Fq).^2;
            obj.q = q1d(:);
            obj.qx = obj.q;
            obj.qy = obj.q;
            obj.qz = obj.q;
        end
        function obj=addElectron(varargin)
            % md.addElectron(rmin, rmax, thresholdvalue, value of electron density) or
            % md.addElectron(rmin, rmax, value of electron density) or
            % md.addElectron(value of electron density)
            % md.addElectron(thresholdvalue, value of electron density)
            % Assuming mean electron density of a protein is max values of
            % the boxes.
            %
            % Usage:
            % For a protein BOX model, adding solvent in boxes whose
            % density is lower than thresholdvalue...
            % Then, subtract the solvent density from entire boxes using
            % addElectron(value) command.
            % Reset the value to the boxes of which values are less than the threshold.
            if numel(varargin)>1
                obj = varargin{1};
            end
            if ndims(obj.Nelectrons) < 2
                error('Dimension of Nelectrons is not 3')
            end
            meanEd = max(max(max(obj.Nelectrons)));
            
            if numel(varargin)==4
                rmin = varargin{2};
                rmax = varargin{3};
                value = varargin{4}*obj.boxvolume;
                rad = sqrt(obj.xbox.^2+obj.ybox.^2+obj.zbox.^2);
                t = (rad >= rmin) & (rad<rmax) & obj.Nelectrons(:)<value;
                obj.Nelectrons(t) = value; 
            elseif numel(varargin)==2
                value = varargin{2}*obj.boxvolume;
                obj.Nelectrons = value;
            elseif numel(varargin) == 3
                thres = varargin{2};
                value = varargin{3}*obj.boxvolume;
                t = obj.Nelectrons < thres;
                obj.Nelectrons(t) = value; 
            end
            obj = obj.averagerho;
        end
        function obj = fillsolvent(varargin)
            % md.fillsolvent(highest_edensity_without_solvent, solvent_edensity)
            % md.fillsolvent(highest_edensity_without_solvent, threshold e_density, solvent_edensity)
            % md.fillsolvent(highest_edensity_without_solvent, rmin, rmax, threshold e_density, solvent_edensity)
            %
            % Usage:
            % Fill solvent in the box where density is lower than the
            % highest_edensity_without_solvent.
            %
            % density is lower than thresholdvalue...
            % Then, subtract the solvent density from entire boxes using
            % addElectron(value) command.
            % Reset the value to the boxes of which values are less than the threshold.
            
            if numel(varargin)>1
                obj = varargin{1};
            end
            if ndims(obj.Nelectrons) < 2
                error('Dimension of Nelectrons is not 3')
            end
            rmin = []; rmax = []; thres = [];
            if numel(varargin) == 3
                IdealDensity = varargin{2};
                solv_density = varargin{3};
            elseif numel(varargin) == 4
                IdealDensity = varargin{2};
                thres = varargin{3};
                solv_density = varargin{4};
            elseif numel(varargin) == 6
                IdealDensity = varargin{2};
                rmin = varargin{3};
                rmax = varargin{4};
                thres = varargin{5};
                solv_density = varargin{6};
            end
            rad = sqrt(obj.xbox.^2+obj.ybox.^2+obj.zbox.^2);

            % adding electrons from water
            t = rad>-1; % make all t 1.
            if ~isempty(rmin)
                t = t & (rad >= rmin) & (rad<rmax); 
            end
            if ~isempty(thres)
                t = t & (obj.Nelectrons(:) < thres*obj.boxvolume);
            end
            t = t & (obj.Nelectrons(:) < IdealDensity*obj.boxvolume);
            %idea_rho_density_protein = 0.4;
            Vx = obj.Nelectrons(t)/IdealDensity;
            WaterElectrons = solv_density*(obj.boxvolume - Vx);
            obj.Nelectrons(t) = obj.Nelectrons(t) + WaterElectrons;
        end

        function obj=makevectq(obj)
            [qx0, qy0, qz0] = meshgrid(obj.qx, obj.qy, obj.qz);
            obj.qv = [qx0(:), qy0(:), qz0(:)];
        end
        function obj=Pqorientavg(obj)
            % orientational average of Pq
            if isempty(obj.qv)
                obj = obj.makevectq;
            end
            qv0 = sqrt(obj.qv(:,1).^2+obj.qv(:,2).^2+obj.qv(:,3).^2);
            qv0 = qv0(:);
            Pqtemp = obj.Pq(:);
            newq = linspace(0, sqrt(2)*max(obj.q), qnum*2);
            qarray = newq-newq(1);
            [h, bin] = histc(qv0, qarray);
            q1d = qarray(1:end-1) + diff(qarray);
            t = bin == 0;
            bin(t) = [];
            Pqtemp(t) = [];

            c=accumarray(bin,Pqtemp);
            %c(end) = [];
            if numel(h)>numel(c)
                h(end) = [];
            end
            
            % to remove the zero occurrences.
            t = h == 0;

            Iqv = c./max(1,h);
            c = Iqv;
            if numel(c) > numel(q1d)
                c(end) = [];
            end
            obj.Pq1d = [q1d(:), c(:)];
            obj.Pq1d(t, :) = [];
        end
        function obj=Pqcal(obj)
            Pq0 = boxPq(obj.q, obj);
            obj.Pq = Pq0;
            %q0 = obj.q;
            %q0(q0<=0) = [];
            rmax = sqrt((obj.cpb{1}(end)-obj.cpb{1}(1))^2+(obj.cpb{2}(end)-obj.cpb{2}(1))^2+(obj.cpb{3}(end)-obj.cpb{3}(1))^2);
            obj.r = linspace(0, rmax*2, 1000);
            obj.pr = Iq2pr(obj.q, Pq0, obj.r);
        end
        function obj = Fqmeancal(obj)
            obj = obj.averagerho;
            obj.Fqmean = rho2Fq(obj.rhoprofile(:,1), obj.rhoprofile(:,2), obj.q);
        end
        function obj = averagerho(obj)
            %rmax = sqrt((obj.cpb{1}(end)-obj.cpb{1}(1))^2+(obj.cpb{2}(end)-obj.cpb{2}(1))^2+(obj.cpb{3}(end)-obj.cpb{3}(1))^2);
            %obj.r = linspace(0, rmax*2, 1000);
            tmp=removeEmptyBox(obj);
            x = tmp.xbox;
            y = tmp.ybox;
            z = tmp.zbox;
            dt = tmp.Nelectrons;
            dt = dt(:);
            
            r0 = sqrt(x.^2+y.^2+z.^2);
            
            %Pqtemp = obj.Pq(:);
            newr = linspace(0, max(r0), 1000);
            qarray = newr-newr(1);
            [h, bin] = histc(r0, qarray);
            r1d = qarray(1:end-1) + diff(qarray);
            t = bin == 0;
            bin(t) = [];
            dt(t) = [];
            
            c=accumarray(bin,dt);
            %c(end) = [];
            if numel(h)>numel(c)
                h(end) = [];
            end
            
            % to remove the zero occurrences.
            t = h == 0;

            c = c./max(1,h);
            if numel(c) > numel(r1d)
                c(end) = [];
            end
            obj.rhoprofile = [r1d(:), c(:)/tmp.boxvolume];            
            obj.rhoprofile(t, :) = [];
        end
        function obj=debyePq(obj)
            [Pq0, r0, pr0] = PDBbox2Pq(obj.q, obj);
            obj.Pq = Pq0;
            obj.r = r0;
            obj.pr = pr0;
        end
        function p = drawiso(obj)
            % to draw the object
            p = patch(obj.isosurf);
            p.FaceColor = 'r';
            p.EdgeColor = 'none';
            p.FaceAlpha = 0.5;
            daspect([1,1,1])
            view(3);
            axis tight
            camlight
            lighting gouraud
        end
        function scatter3(obj)
            % to draw the object
            obj.removeEmptyBox
            scatter3(obj.xbox(:), obj.ybox(:), obj.zbox(:),[], obj.Nelectrons(:))
            daspect([1,1,1])
            view(3);
        end
    end
end
