function [Pq, Fqm, pr, Fqos] = boxPq(q, BOX, dim)
% [Pq, Fqm] = boxPq(q, pdb, dim)
% calculate SAXS P(q) from pdb and <Fq>_orient
% Look also pdb_Fq.m

% Byeongdu Lee
if nargin < 3
    dim = 3; % 3D random orientation
end

isbox = false;

if isa(BOX, 'BOXmodel')
    isbox = true;
    atom = BOX.Nelectrons;
    x = BOX.X;
    y = BOX.Y;
    z = BOX.Z;
    if isempty(x)
        x = BOX.xbox;
        z = BOX.ybox;
        y = BOX.zbox;
    end
elseif isa(BOX, 'PDB')
    isbox = false;
    atom = BOX.Atom;
    x = BOX.X;
    y = BOX.Y;
    z = BOX.Z;
end


if ~isbox
    atmN = unique(atom);
    maxAtmNumber = max(atmN);
    atmFq = zeros(1, maxAtmNumber);

    if max(q)<1
        atmFq(atmN) = atmN;
    else
        atmFq = [];
    end

    if isempty(atmFq)
        qv = q;
        if size(q, 2) ==2
            qv = sqrt(q(:,1).^2+q(:,2).^2);
        elseif size(q, 2) == 3
            qv = sqrt(q(:,1).^2+q(:,2).^2+q(:,3).^2);
        end
        atmFq = zeros(numel(qv), maxAtmNumber);
        for i=1:numel(atmN)
            atmFq(:,atmN(i)) = atomicscatteringfactor(qv, atmN(i));
        end
    end
    
    if nargout == 1
        switch dim
            case 3
                Pq = debyem(q, x, y, z, atom);
                %Pq = debyepr(q, x, y, z, atom);
                %Iq = Iq(:,1)-size(pv, 1);
            case 2
                Pq = debye2d(q, x, y, z, atom);
        end
        return
    end
end

% if r and pr are available, use them for P(q) calculation.
Pq = [];
Fqos = [];
if (dim == 3) 
    if ~isempty(BOX.pr)
        Pq = pr2Iq(BOX.r, BOX.pr, q);
%     else
%         [Pq, r, pr] = debyepr(q, x, y, z, 1/max(q));
%         BOX.r = r;
%         BOX.pr = pr;
    end
    if nargout > 1
        %Fqm = ones(size(Pq));
        Fqos = [];
        if isprop(BOX, 'rhoprofile')
            if ~isempty(BOX.rhoprofile)
                Fqos = rho2Fq(BOX.rhoprofile(:,1),BOX.rhoprofile(:,2), q);
            end
        end
        if isempty(Fqos)
            BOX.q = q;
            BOX = BOX.Fqmeancal;
            Fqos = BOX.Fqmean(:,2); 
        else
            Fqos = sign(Fqos(:)).*sqrt(Pq);
        end
    end

end
pr = [];Fqm = Fqos;
if ~isempty(Pq) && ~isempty(Fqos)
    return
end
if (nargout == 1) 
    if ndims(BOX.Nelectrons) > 0
        obj = BOX.removeEmptyBox;
        ed = obj.Nelectrons(:)';
        x = obj.xbox;
        y = obj.ybox;
        z = obj.zbox;
        boxsize = obj.resolution;
        % only one value of density;
        if numel(unique(ed)) == 1
            Pq = debyepr(q, x, y, z, boxsize);
        else
            % when there are multiple electronds.
            [Fq0, atm] = unique(ed);
            Fq = ones(numel(q), numel(atm));
            for i=1:numel(atm)
                Fq(:,i) = Fq(:,i) * Fq0(i);
            end
            Pq = debyepr(q, x, y, z, boxsize, atm, Fq);
        end
        return
    end
end

% otherwise, compute and average with spiral method.
if numel(x) > 5000
    N2 = 2^6;
else
    N2 = 2^7;
end
%N2 = N^2;
MAXpntat1compute = N2/1; % This number provides the fast calculation.
[Qx, Qy, Qz] = spiral_on_sphere(1, N2);

%if N^2*nrp*numq > MAXpntat1compute
if size(q,1)==1 && size(q,2)>1
    q = q(:);
end
QQ = q;
numq = numel(q(:,1));

qseg = round(N2/MAXpntat1compute);
NumQseg = round(N2/qseg);
% qseg = round(N2/NumQseg);

Pq = zeros(size(q));
Fqm = zeros(size(q));
tic
percentprogress = 10;
for seg=1:numq
    q = QQ(seg);
    qx = q*Qx';
    qy = q*Qy';
    qz = q*Qz';
    Fq = zeros(1, N2);
    for qxseg = 1:qseg
        if qxseg < qseg
            ind = (qxseg-1)*NumQseg+1:NumQseg*qxseg;
        else
            ind = (qxseg-1)*NumQseg+1:N2;
        end
        qxs = qx(ind);
        qys = qy(ind);
        qzs = qz(ind);
        
%    sizeqx = size(qx);
        qxs = qxs(:);
        qys = qys(:);
        qzs = qzs(:);
        ff = fq_box(qxs, qys, qzs, BOX);
        Fq(ind) = ff(:)';
        Iqt = mean(abs(Fq).^2);
        if nargout ==2
            Fqt = mean(Fq);
        end
        if (seg/numq*100 > percentprogress)
            fprintf('%0.3f percent done.\n', seg/numq*100)
            percentprogress = percentprogress + 10;
        end
    end
    Pq(seg) = Iqt;
    if nargout == 2
        Fqm(seg) = Fqt;
    end
end
toc    

