% Author : Gerhard Nieuwenhuiys.
% Date of last revision : 2009/02/02.
% Thank you for downloading / using Molpackage V2. Please contact me at
% gerhardn654@yahoo.com should you have any enquiries or suggestions for
% improvement.
% To view data on an element use pertable(a),where a is the atomic
% number of the element or its corresponding symbol.
% Interpretation of output:
% Melting Point in C
% Boiling Point in C : eg.(s2550) means that sublimation occurs at 2550C
% Density in kg/m3
% Isotope data : eg. '95.02%32 0.75%33 4.21%34 0.02%36' means that the
% naturally occuring element constitutes 95.02% of isotope mass 32 and
% 0.75% of isotope mass 33, etc.
% Please note that the abbreviation, NA means :Not Available.
% See also the related functions : sym2an , molecules2
%
% List of Elements and Atomic Numbers:
% 1(H)        2(He)       3(Li)       4(Be)       5(B)
% 6(C)        7(N)        8(O)        9(F)        10(Ne)
% 11(Na)      12(Mg)      13(Al)      14(Si)      15(P)
% 16(S)       17(Cl)      18(Ar)      19(K)       20(Ca)
% 21(Sc)      22(Ti)      23(V)       24(Cr)      25(Mn)
% 26(Fe)      27(Co)      28(Ni)      29(Cu)      30(Zn)
% 31(Ga)      32(Ge)      33(As)      34(Se)      35(Br)
% 36(Kr)      37(Rb)      38(Sr)      39(Yt)      40(Zr)
% 41(Nb)      42(Mo)      43(Tc)      44(Ru)      45(Rh)
% 46(Pd)      47(Ag)      48(Cd)      49(In)      50(Sn)
% 51(Sb)      52(Te)      53(I)       54(Xe)      55(Cs)
% 56(Ba)      57(La)      58(Ce)      59(Pr)      60(Nd)
% 61(Pm)      62(Sm)      63(Eu)      64(Gd)      65(Tb)
% 66(Dy)      67(Ho)      68(Er)      69(Tm)      70(Yt)
% 71(Lu)      72(Hf)      73(Ta)      74(W)       75(Re)
% 76(Os)      77(Ir)      78(Pt)      79(Au)      80(Hg)
% 81(Tl)      82(Pb)      83(Bi)      84(Po)      85(At)
% 86(Rn)      87(Fr)      88(Ra)      89(Ac)      90(Th)
% 91(Pa)      92(U)       93(Np)      94(Pu)      95(Am)
% 96(Cm)      97(Bk)      98(Cf)      99(Es)      100(Fm)
%
% Example 1: >> pertable(6)
% results in : AtomicNumber: 6
%               Element: 'Carbon'
%                Symbol: 'C'
%          MeltingPoint: '3727'
%          BoilingPoint: '4830'
%               Density: '2250'
%                RAmass: '12.011'
%       NaturalIsotope1: '98.9%12 1.1%13'
%       NaturalIsotope2: ''
%       NaturalIsotope3: ''
%       OxidationStates: '-4,2,4'
%     ElectroNegativity: '2.5'
% Example 2: If you do not know the atomic number and the help file isn't
% at hand you can use sym2an to convert chemical symbols to atomic numbers.
% >> pertable(sym2an('C')) yields the same result as above.
% Example 3: Additional to this software you can also use the symbols
% directly.
% >> pertable('Au') results in:
%          AtomicNumber: 79
%               Element: 'Gold'
%                Symbol: 'Au'
%          MeltingPoint: '1063'
%          BoilingPoint: '2970'
%               Density: '19320'
%                RAmass: '196.9665'
%       NaturalIsotope1: '100%197'
%       NaturalIsotope2: ''
%       NaturalIsotope3: ''
%       OxidationStates: '1,3'
%     ElectroNegativity: '1.4'
% Example 4: You may also use the chemical symbol as a non-string input, but
% note that this will require you to load the mat-file atoms.mat before hand.
% This is simply done by typing >> load atoms in the workspace. The
% following is an illustration.
% >> load atoms
% >> pertable(S)
%          AtomicNumber: 16
%               Element: 'Sulfur'
%                Symbol: 'S'
%          MeltingPoint: '119'
%          BoilingPoint: '445'
%               Density: '2070'
%                RAmass: '32.06'
%       NaturalIsotope1: '95.02%32 0.75%33 4.21%34'
%       NaturalIsotope2: '0.02%36'
%       NaturalIsotope3: ''
%       OxidationStates: '-2,2,4,6'
%     ElectroNegativity: '2.4'
% References : All data in this mfile were taken from the literature.
% 1. Fisika 1ste uitgawe 1992,ISBN 0 627 01787 8
% Outeurs : JC van Staden, R Swanepoel, JW Koen.
% 2. Chemical Principles 4th ed 1977,ISBN 0 7216 6163 7
% Authors : WL Masterton, I Slowinski.
% 3. MERCK, Periodic Table of the Elements.
% 4. P. De Bievre and P.D.P. Taylor, Int. J. Mass Spectrom. Ion Phys. 123,
% 149 (1993). As per IUPAC recommendation.
% .........................................................................
function pertable(element);
AtomicNumber={1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,...
    38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,...
    75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,...
    109,'Not Referenced'};
Element={'Hydrogen','Helium','Lithium','Beryllium','Boron','Carbon','Nitrogen','Oxygen','Fluorine','Neon','Sodium',...
    'Magnesium','Aluminium','Silicon','Phosphorus','Sulfur','Chlorine','Argon','Potassium','Calcium','Scandium',...
    'Titanium','Vanadium','Chromium','Manganese','Iron','Cobalt','Nickel','Copper','Zinc','Gallium','Germanium',...
    'Arsenic','Selenium','Bromine','Krypton','Rubidium','Strontium','Yttrium','Zirconium','Niobium','Molybdenum',...
    'Technetium','Ruthenium','Rhodium','Palladium','Silver','Cadmium','Indium','Tin','Antimony','Tellurium',...
    'Iodine','Xenon','Caesium','Barium','Lanthanum','Cerium','Praseodynium','Neodymium','Promethium','Samarium',...
    'Europium','Gadolinium','Terbium','Dysprosium','Holmium','Erbium','Thulium','Ytterbium','Lutetium','Hafnium',...
    'Tantalum','Tungsten','Rhenium','Osmium','Iridium','Platinum','Gold','Mercury','Thallium','Lead','Bismuth',...
    'Polonium','Astatine','Radon','Francium','Radium','Actinium','Thorium','Protactinium','Uranium','Neptunium',...
    'Plutonium','Americium','Curium','Berkelium','Californium','Einsteinium','Fermium','Mendelevium','Nobelium',...
    'Lawrencium','Rutherfordium','Dubnium','Seaborgium','Bohrium','Hassium','Meitnerium','#'};
Symbol={'H','He','Li','Be','B','C','N','O','F','Ne','Na','Mg','Al','Si','P','S','Cl','Ar','K','Ca','Sc','Ti','V',...
    'Cr','Mn','Fe','Co','Ni','Cu','Zn','Ga','Ge','As','Se','Br','Kr','Rb','Sr','Yt','Zr','Nb','Mo','Tc','Ru','Rh',...
    'Pd','Ag','Cd','In','Sn','Sb','Te','I','Xe','Cs','Ba','La','Ce','Pr','Nd','Pm','Sm','Eu','Gd','Tb','Dy','Ho'...
    'Er','Tm','Yt','Lu','Hf','Ta','W','Re','Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn','Fr','Ra','Ac',...
    'Th','Pa','U','Np','Pu','Am','Cm','Bk','Cf','Es','Fm','Md','No','Lr','Rf','Db','Sg','Bh','Hs','Mt','#'};
MeltingPoint={'-259','-270','180','1277','2030','3727','-210','-219','-220','-249','98','650','660','1410','44',...
    '119','-101','-189','64','838','1539','1668','1900','1875','1245','1536','1495','1453','1083','420','30','937',...
    '817','217','-7','-157','39','768','1509','1852','2468','2610','2130','2500','1966','1552','961','321','156',...
    '232','630','450','114','-112','287','714','920','804','919','1019','1027','1072','826','1312','1356','1407',...
    '1461','1497','1545','824','1652','2222','2996','3410','3180','2700','2454','1769','1063','-38','303','327',...
    '271','254','302','-71','20','700','1050','1750','1552','1132','640','641','994','1340','986','864','860',...
    'NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','#'};
BoilingPoint={'-253','-269','1330','2770','s2550','4830','-196','-183','-188','-246','892','1107','2450','2680',...
    '280','445','-35','-186','760','1440','2730','3260','3460','2665','2150','3000','2900','2730','2595','906',...
    '2237','2830','s613','685','58','-152','688','1380','3030','3580','4927','5560','NA','4900','4500','3980','2210',...
    '765','2000','2270','1380','990','183','-108','690','1640','3470','3470','3020','3180','NA','1630','1490','2730',...
    '2530','2330','2330','2630','1720','1530','1930','5400','5425','5930','5900','5500','5300','4530','2970','357',...
    '1457','1725','1560','962','337','-62','657','1140','3200','3850','4227','3818','3902','3232','2607','1340','NA',...
    'NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','#'};
Density={'0.09','0.18','530','1850','2340','2250','1.25','1.43','1.7','0.9','970','1740','2700','2330','1830','2070',...
    '3.21','1.78','860','1550','2990','4510','6100','7190','7430','7870','8850','8900','8960','7130','5910','5320',...
    '5720','4790','3120','3.74','1530','2600','4470','6490','8570','10220','11490','12200','12440','12020','10490',...
    '8650','7310','7300','6620','6240','4940','5.9','1900','3500','6190','6770','6770','7000','NA','7490','5240',...
    '7860','8250','8550','6790','9150','9310','6960','9850','13090','16600','19300','21040','22570','22500','21450',...
    '19320','13550','11850','11360','9800','9200','NA','9.96','2410','5000','10070','11660','15370','19070','20450',...
    '19840','13670','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','#'};
RAM={'1.0079','4.0026','6.941','9.01218','10.81','12.011','14.0067','15.9994','18.9984','20.179','22.98977','24.305',...
    '26.98154','28.086','30.97376','32.06','35.453','39.948','39.098','40.08','44.9559','47.9','50.9414','51.996',...
    '54.938','55.847','58.9332','58.7','63.546','65.38','69.72','72.59','74.9216','78.96','79.904','83.8','85.4678',...
    '87.62','88.9059','91.22','92.9064','95.94','98.9062','101.07','102.9055','106.4','107.868','112.4','114.82',...
    '118.69','121.75','127.6','126.9045','131.3','132.9054','137.34','138.9055','140.12','140.9077','144.24','147',...
    '150.4','151.96','157.25','158.9254','162.5','164.9304','167.26','168.9342','173.04','174.97','178.49','180.9479',...
    '183.85','186.207','190.2','192.22','195.09','196.9665','200.59','204.37','207.2','208.9804','210','210','222',...
    '223','226.0254','227','232.0381','231.0359','238.029','237.0482','244','243','247','247','251','254','257','NA',...
    'NA','NA','NA','NA','NA','NA','NA','NA','#'};
a=['99.985%1 0.015%2'];b=['0.000137%3 99.999863%4'];c=['7.5%6 92.5%7'];d=['100%9'];e=['19.9%10 80.1%11'];
f=['98.9%12 1.1%13'];g=['99.634%14 0.366%15'];h=['99.762%16 0.038%17 0.2%18'];j=['100%19'];k=['90.48%20 0.27%21 9.25%22'];
l=['100%23'];m=['78.99%24 10%25 11.01%26'];n=['100%27'];o=['92.23%28 4.67%29 3.1%30'];p=['100%31'];q=['95.02%32 0.75%33 4.21%34'];
r=['75.77%35 24.23%37'];s=['0.337%36 0.063%38 99.6%40'];u=['93.2581%39 0.0117%40'];v=['96.941%40 0.647%42 0.135%43'];
w=['100%45'];x=['8%46 7.3%47 73.8%48 5.5%49'];y=['0.25%50 99.75%51'];z=['4.345%50 83.789%52'];aa=['100%55'];
ab=['5.8%54 91.72%56 2.2%57'];ac=['100%59'];ad=['68.077%58 26.223%60 1.14%61'];ae=['69.17%63 30.83%65'];af=['48.6%64 27.9%66 4.1%67'];
ag=['60.108%69 39.892%71'];ah=['21.23%70 27.66%72 7.73%73'];aj=['100%75'];ak=['0.89%74 9.36%76 7.63%77'];al=['50.69%79 49.31%81'];
am=['0.35%78 2.25%80 11.6%82'];an=['72.165%85 27.835%87'];ao=['0.56%84 9.86%86 7%87'];ap=['100%89'];aq=['51.45%90 11.22%91 17.15%92'];
ar=['100%93'];as=['14.84%92 9.25%94 15.92%95'];au=['No Stable Isotope'];av=['5.52%96 1.88%98 12.7%99'];aw=['100%103'];
ax=['1.02%102 11.14%104'];ay=['51.839%107 48.161%109'];az=['1.25%106 0.89%108 12.49%110'];ba=['4.3%113 95.7%115'];
bb=['0.97%112 0.65%114 0.34%115'];bc=['57.36%121 42.64%123'];bd=['0.096%120 2.603%122 0.908%123'];be=['100%127'];
bf=['0.1%124 0.09%126 1.91%128'];bg=['100%133'];bh=['0.106%130 0.101%132 2.417%134'];bj=['0.0902%138 99.9098%139'];
bk=['0.19%136 0.25%138 88.48%140'];bl=['100%141'];bm=['27.13%142 12.18%143 23.8%144'];bn=['NA'];bo=['3.1%144 15%147 11.3%148'];
bp=['47.8%151 52.2%153'];bq=['0.2%152 2.18%154 14.8%155'];br=['100%159'];bs=['0.06%156 0.1%158 2.34%160'];bu=['100%165'];
bv=['0.14%162 1.61%164 33.6%166'];bw=['100%169'];bx=['0.13%168 3.05%170 14.3%171'];by=['97.41%175 2.59%176'];
bz=['0.162%174 5.206%176 18.606%177'];ca=['0.012%180 99.988%181'];cb=['0.13%180 26.3%182 14.3%183'];cc=['37.4%185 62.6%187'];
cd=['0.02%184 1.58%186 1.6%187'];ce=['37.3%191 62.7%193'];cf=['0.01%190 0.79%192 32.9%194'];cg=['100%197'];
ch=['0.15%196 9.97%198 16.87%199'];cj=['29.524%203 70.476%205'];ck=['1.4%204 24.1%206 22.1%207'];cl=['100%209'];cm=['NA'];
cn=['NA'];co=['NA'];cp=['NA'];cq=['NA'];cr=['NA'];cs=['100%232'];cu=['NA'];cv=['0.0055%234 0.72%235'];cw=['NA'];cx=['NA'];
cy=['NA'];cz=['NA'];da=['NA'];db=['NA'];dc=['NA'];dd=['NA'];de=['NA'];df=['NA'];dg=['NA'];dh=['NA'];dj=['NA'];dk=['NA'];
dl=['NA'];dm=['NA'];dn=['NA'];do=['#'];
Isotope={a,b,c,d,e,f,g,h,j,k,l,m,n,o,p,q,r,s,u,v,w,x,y,z,aa,ab,ac,ad,ae,...
    af,ag,ah,aj,ak,al,am,an,ao,ap,aq,ar,as,au,av,aw,ax,ay,az,ba,bb,bc,bd,...
    be,bf,bg,bh,bj,bk,bl,bm,bn,bo,bp,bq,br,bs,bu,bv,bw,bx,by,bz,ca,cb,cc,...
    cd,ce,cf,cg,ch,cj,ck,cl,cm,cn,co,cp,cq,cr,cs,cu,cv,cw,cx,cy,cz,da,db,...
    dc,dd,de,df,dg,dh,dj,dk,dl,dm,dn,do};
a2=[''];b2=[''];c2=[''];d2=[''];e2=[''];f2=[''];g2=[''];h2=[''];j2=[''];k2=[''];l2=[''];m2=[''];n2=[''];o2=[''];p2=[''];
q2=['0.02%36'];r2=[''];s2=[''];u2=['6.7302%41'];v2=['2.086%44 0.004%46 0.187%48'];w2=[''];x2=['5.4%50'];y2=[''];
z2=['9.501%53 2.365%54'];aa2=[''];ab2=['0.28%58'];ac2=[''];ad2=['3.634%62 0.926%64'];ae2=[''];af2=['18.8%68 0.6%70'];
ag2=[''];ah2=['35.94%74 7.44%76'];aj2=[''];ak2=['23.78%78 49.61%80 8.73%82'];al2=[''];am2=['11.5%83 57%84 17.3%86'];
an2=[''];ao2=['82.58%88'];ap2=[''];aq2=['17.38%94 2.8%96'];ar2=[''];as2=['16.68%96 9.55%97 24.13%98'];au2=[''];
av2=['12.6%100 17%101 31.6%102'];aw2=[''];ax2=['22.33%105 27.33%106 26.46%108'];ay2=[''];az2=['12.8%111 24.13%112 12.22%113'];
ba2=[''];bb2=['14.53%116 7.68%117 24.23%118'];bc2=[''];bd2=['4.816%124 7.139%125 18.95%126'];be2=[''];
bf2=['26.4%129 4.1%130 21.2%131'];bg2=[''];bh2=['6.592%135 7.854%136 11.23%137'];bj2=[''];bk2=['11.08%142'];bl2=[''];
bm2=['8.3%145 17.19%146 5.76%148'];bn2=[''];bo2=['13.8%149 7.4%150 26.7%152'];bp2=[''];bq2=['20.47%156 15.65%157 24.84%158'];
br2=[''];bs2=['18.9%161 25.5%162 24.9%163'];bu2=[''];bv2=['22.95%167 26.8%168 14.9%170'];bw2=[''];bx2=['21.9%172 16.12%173 31.8%174'];
by2=[''];bz2=['27.297%178 13.629%179 35.1%180'];ca2=[''];cb2=['30.67%184 28.6%186'];cc2=[''];cd2=['13.3%188 16.1%189 26.4%190'];
ce2=[''];cf2=['33.8%195 25.3%196 7.2%198'];cg2=[''];ch2=['23.1%200 13.18%201 29.86%202'];cj2=[''];ck2=['52.4%208'];
cl2=[''];cm2=[''];cn2=[''];co2=[''];cp2=[''];cq2=[''];cr2=[''];cs2=[''];cu2=[''];cv2=['99.2745%238'];cw2=[''];cx2=[''];
cy2=[''];cz2=[''];da2=[''];db2=[''];dc2=[''];dd2=[''];de2=[''];df2=[''];dg2=[''];dh2=[''];dj2=[''];dk2=[''];dl2=[''];
dm2=[''];dn2=[''];do2=[''];
Isotope2={a2,b2,c2,d2,e2,f2,g2,h2,j2,k2,l2,m2,n2,o2,p2,q2,r2,s2,u2,v2,w2,...
    x2,y2,z2,aa2,ab2,ac2,ad2,ae2,af2,ag2,ah2,aj2,ak2,al2,am2,an2,ao2,ap2,...
    aq2,ar2,as2,au2,av2,aw2,ax2,ay2,az2,ba2,bb2,bc2,bd2,be2,bf2,bg2,bh2,...
    bj2,bk2,bl2,bm2,bn2,bo2,bp2,bq2,br2,bs2,bu2,bv2,bw2,bx2,by2,bz2,ca2,...
    cb2,cc2,cd2,ce2,cf2,cg2,ch2,cj2,ck2,cl2,cm2,cn2,co2,cp2,cq2,cr2,cs2,...
    cu2,cv2,cw2,cx2,cy2,cz2,da2,db2,dc2,dd2,de2,df2,dg2,dh2,dj2,dk2,dl2,...
    dm2,dn2,do2};
a3=[''];b3=[''];c3=[''];d3=[''];e3=[''];f3=[''];g3=[''];h3=[''];j3=[''];k3=[''];l3=[''];m3=[''];n3=[''];o3=[''];p3=[''];
q3=[''];r3=[''];s3=[''];u3=[''];v3=[''];w3=[''];x3=[''];y3=[''];z3=[''];aa3=[''];ab3=[''];ac3=[''];ad3=[''];ae3=[''];
af3=[''];ag3=[''];ah3=[''];aj3=[''];ak3=[''];al3=[''];am3=[''];an3=[''];ao3=[''];ap3=[''];aq3=[''];ar3=[''];as3=['9.63%100'];
au3=[''];av3=['18.7%104'];aw3=[''];ax3=['11.72%110'];ay3=[''];az3=['28.73%114 7.49%116'];ba3=[''];
bb3=['8.59%119 32.59%120 4.63%122 5.79%124'];bc3=[''];bd3=['31.69%128 33.8%130'];be3=[''];bf3=['26.9%132 10.4%134 8.9%136'];
bg3=[''];bh3=['71.7%138'];bj3=[''];bk3=[''];bl3=[''];bm3=['5.64%150'];bn3=[''];bo3=['22.7%154'];bp3=[''];bq3=['21.86%160'];
br3=[''];bs3=['28.2%164'];bu3=[''];bv3=[''];bw3=[''];bx3=['12.7%176'];by3=[''];bz3=[''];ca3=[''];cb3=[''];cc3=[''];
cd3=['41%192'];ce3=[''];cf3=[''];cg3=[''];ch3=['6.87%204'];cj3=[''];ck3=[''];cl3=[''];cm3=[''];cn3=[''];co3=[''];
cp3=[''];cq3=[''];cr3=[''];cs3=[''];cu3=[''];cv3=[''];cw3=[''];cx3=[''];cy3=[''];cz3=[''];da3=[''];db3=[''];dc3=[''];
dd3=[''];de3=[''];df3=[''];dg3=[''];dh3=[''];dj3=[''];dk3=[''];dl3=[''];dm3=[''];dn3=[''];do3=[''];
Isotope3={a3,b3,c3,d3,e3,f3,g3,h3,j3,k3,l3,m3,n3,o3,p3,q3,r3,s3,u3,v3,w3,x3,...
    y3,z3,aa3,ab3,ac3,ad3,ae3,af3,ag3,ah3,aj3,ak3,al3,am3,an3,ao3,ap3,aq3,...
    ar3,as3,au3,av3,aw3,ax3,ay3,az3,ba3,bb3,bc3,bd3,be3,bf3,bg3,bh3,bj3,bk3,...
    bl3,bm3,bn3,bo3,bp3,bq3,br3,bs3,bu3,bv3,bw3,bx3,by3,bz3,ca3,cb3,cc3,cd3,...
    ce3,cf3,cg3,ch3,cj3,ck3,cl3,cm3,cn3,co3,cp3,cq3,cr3,cs3,cu3,cv3,cw3,cx3,...
    cy3,cz3,da3,db3,dc3,dd3,de3,df3,dg3,dh3,dj3,dk3,dl3,dm3,dn3,do3};
Oxidation={'-1,1','NA','1','2','3','-4,2,4','-3,2,3,4,5','-2,-1','-1','NA',...
    '1','2','3','4','-3,3,5','-2,2,4,6','-1,1,3,5,7','NA','1','2',...
    '3','3,4','2,3,4,5','2,3,6','2,3,4,6,7','2,3,6','2,3','2,3','1,2','2',...
    '3','4','-3,3,5','-2,4,6','-1,1,3,5,7','2,4','1','2','3','4',...
    '3,5','2,3,4,5,6','7','3,4,8','1,2,3,4','2,4','1,2','2','3','2,4',...
    '-3,3,5','-2,4,6','-1,1,3,5,7','2,4,6','1','2','3','3,4','3,4','3',...
    '3','2,3','2,3','3','3,4','3','3','3','2,3','2,3',...
    '3','4','5','2,3,4,5,6','2,4,7','2,3,4,6,8','1,2,3,4,6','2,4','1,3','1,2',...
    '1,3','2,4','3,5','2,4,6','-1,1,3,5,7','2','1','2','3','4',...
    '4,5','3,4,5,6','3,4,5,6','3,4,5,6','3,4,5,6','3,4','3,4','3,4','3','3','NA',...
    'NA','NA','NA','NA','NA','NA','NA','NA','#'};
Elneg={'2.2','NA','1','1.5','2','2.5','3.1','3.5','4.1','NA',...
    '1','1.2','1.5','1.7','2.1','2.4','2.8','NA','0.9','1',...
    '1.2','1.3','1.5','1.6','1.6','1.6','1.7','1.8','1.8','1.7',...
    '1.8','2','2.2','2.5','2.7','NA','0.9','1','1.1','1.2',...
    '1.2','1.3','1.4','1.4','1.5','1.4','1.4','1.5','1.5','1.7',...
    '1.8','2','2.2','NA','0.9','1','1.1','1.1','1.1','1.1',...
    '1.1','1.1','1','1.1','1.1','1.1','1.1','1.1','1.1','1.1',...
    '1.1','1.2','1.3','1.4','1.5','1.5','1.6','1.4','1.4','1.5',...
    '1.4','1.6','1.7','1.8','2','NA','0.9','1','1','1.1',...
    '1.1','1.2','1.2','1.2','~1.2','~1.2','~1.2','~1.2','~1.2','~1.2',...
    'NA','NA','NA','NA','NA','NA','NA','NA','NA','#'};
pertable=struct('AtomicNumber',AtomicNumber,'Element',Element,'Symbol',Symbol,'MeltingPoint',MeltingPoint,...
    'BoilingPoint',BoilingPoint,'Density',Density,'RAmass',RAM,'NaturalIsotope1',Isotope,'NaturalIsotope2',...
    Isotope2,'NaturalIsotope3',Isotope3,'OxidationStates',Oxidation,'ElectroNegativity',Elneg);
if ischar(element)==1;
    y=load('atoms',element);
    tf=isfield(y,element);
    if tf==1;
        t=y.(element);
    elseif tf==0;
        disp(['!ERROR! Your input could not be processed                  ';...
            'Please verify your request or consult alternative resources']);
        t=110;
    end;
else;
    t=element;
end;
if element>109;
    t=110;
end;
disp(pertable(t));