% This is a class to describe a molecule.
% It inherits atomic properties from the class atom.
% One can define a molecule from a chemical formula, a list of names of
% atoms, and a list of atomic symbols.
% Once a molecule is constructed, Mw will be calculated.
% Coordinates of atoms can be later added as the fields named, X, Y and Z.
% Then, one can draw the molecule using the method, draw.
%   for example: 
%   a = molecule('xxx');
%   a.X = [....];
%   a.Y = [....];
%   a.Z = [....];
%   a.draw
%
% This use Jeff Kantor's parse_formula.m in StoichTool.
%
% Byeongdu Lee
% 12/25/2014

classdef molecule < handle & atom
%     properties (Hidden)
%     AN_
%     ElementName_
%     Sym_
%     Radius_
%     Color_
%     RAMass_
%     end
    properties (SetAccess = public)
        Atom
        Formula
        Element
        Mw
        X
        Y
        Z
    end
    methods
        function obj = molecule(varargin)
            % atomic number : [ 1, 5, 10]....
            % copy the Hidden properties from the class atom.
            mco = ?molecule;
            plist = mco.PropertyList;
            a = atom;
            ishidden = [plist.Hidden];
            fn = {plist.Name};
            fn = fn(ishidden);
            for i=1:numel(fn)
                obj.(fn{i}) = a.(fn{i});
            end
            
            % construct a molecule.....................
            % When the input is empty,
            if isempty(varargin)
                obj.Atom = [];
                obj.Element = [];
                return;
            end
            % When the input is a list of atomic numbers,
            if isnumeric(varargin{1})
                obj.Atom = varargin{1};
                obj.updatefromAtomicN;
                return
            % When the input is a chemical formula,    
            elseif ischar(varargin{1})
                if size(varargin{1},1) == 1
                    obj.Formula = varargin{1};
                    obj.parse_CF;
                    obj.updatefromAtomicN;
                    return;
                end
                obj.Element = varargin{1};
                obj.getAtomicN;
            % When input is a list of atomic symbols,    
            elseif iscell(varargin{1})
                obj.Element = varargin{1};
                obj.getAtomicN;
            end
            obj.getMw;
            obj.getCF;
            %obj = obj.updatefromAtomicN;
        end
        function getAtomicN(obj)
            mthd = 'atomName';
            if isprop(obj, 'Element')
                if iscell(obj.Element)
                    if ~isempty(obj.Element{1})
                        mthd = 'Element';
                    end
                else
                    if ~isempty(obj.Element)
                        mthd = 'Element';
                    end
                end
            end
            
            obj.Atom = S2AN(obj, obj.(mthd));
        end
        function getElm(obj)
            obj.Element = AN2S(obj, obj.Atom);
        end
        function AN = S2AN(obj, S)
            if isempty(S)
                error('Does not contain any symbol\n');
            end
            AN = zeros(size(S));
            for i=1:numel(S)
                try
                    AN(i) = findcellstr(obj.Sym_, S(i));
                catch
                    errstr = sprintf('The element name %s is not found in the molecule class.\n', S{i});
                    error(errstr); %#ok<SPERR>
                end
            end
        end
        function S = AN2S(obj, AN)
            if isempty(AN)
                error('Does not contain any symbol\n');
            end
            S = obj.Sym_(AN);
        end
        function getMw(obj)
            if isempty(obj.Atom)
                obj.getAtomicN;
            end
            obj.Mw = sum(obj.RAMass_(obj.Atom));
        end
        function obj = getCF(obj)
            if isempty(obj.Atom)
                obj.getAtomicN;
            end
            [a, ~, c] = unique(obj.Atom);
            N = hist(c, numel(a));
            str = '';
            for i=1:numel(N)
                if N(i) == 1
                    obj.Sym_(a(i))
                    str = sprintf('%s%s', str, obj.Sym_{a(i)});
                else
                    str = sprintf('%s%s%i', str, obj.Sym_{a(i)}, N(i));
                end
            end
            obj.Formula = str;
        end
        function obj = parse_CF(obj)
            a = parse_formula(obj.Formula);
            fn = fieldnames(a);
            %obj.Element = fn;
            N = cell2mat(struct2cell(a));
            AtN = cell(1, sum(N));
            k = 1;
            for i=1:numel(N)
                AtN(k:(k+N(i)-1)) = fn(i);
                k = k + N(i);
            end
            obj.Element = AtN;
            obj.getAtomicN;
            obj.getMw;
        end
        function updatefromAtomicN(obj)
            obj.getElm;
            obj.getMw;
            obj.getCF;
            obj.getRadius;
            obj.getColor;
        end
        function draw(varargin)
            if numel(varargin)==1
                obj = varargin{1};
            end
            if isempty(findobj(gcf, 'type', 'light'))
                %light;
                light('Position',[-1 -1 -2]);
            end
            hold on;
            [x,y,z] = sphere(10);
            if ~isprop(obj, 'X')
                error('No atomic coordinates.\n')
            end
            if numel(obj.X)<100000
                for i=1:numel(obj.X)
                    r = obj.Radius_(obj.Atom(i));
                    color = obj.Color_(obj.Atom(i), :);hold on;
                    surface('XData',obj.X(i) + r*x,'YData',obj.Y(i) + r*y,...
                        'ZData',obj.Z(i) + r*z,'FaceColor',color,...
                        'EdgeColor','none','FaceLighting','gouraud')
                end
            else
                plot3(obj.X, obj.Y, obj.Z, 'o', 'markersize', 0.5);
            end
            axis image;
        end
    end
end
