function [FF, vol] = saxs_spiralcylinder(qx, qy, qz, param)
% [FF, vol] = saxs_spiralcylinder(qx, qy, qz, [Rh, R, ph, N])
% Calculate the form factor of a helical cylinder.
% q, psi, and chi are m x 1 vector for q vectors.
% chi is the angle between the helic axis to the q vector.
% psi is the in-plane angle between q and the helix.
% A helical cylinder is defined with Rh, R, ph, N
% Rh : Radius of the helix in [nm or angstrom]
% R : Radius of the cylinder in [nm or angstrom]
% ph : pitch distance in [nm or angstrom]
% N : number of pitch
Rh = param(1);
R = param(2);
ph = param(3);
N = param(4);

if Rh > 0
    v = ph/(2*pi*Rh);
    vnz = (1+v^2)^(-1/2);
    Rh_vnz = Rh/vnz;
    helixlength = 2*pi*Rh_vnz;
else
    vnz = 0;
    Rh_vnz = ph/(2*pi);
    helixlength = ph;
    v = ph/(2*pi);
end
%vn = (v^2+1)^(1/2);
if ~isreal(vnz)
    error('The pitch length ph should be at least sqrt(2*pi)*Rh')
end
% fprintf('Length of a turn of the helix is %0.5e.\n', helixlength);
% fprintf('Volume of a turn of the helix is %0.5e.\n', helixlength*pi*R^2);
vol = helixlength*pi*R^2;
Nh = 500; % number of points to make a pitch.
t = linspace(0, 2*pi, Nh+1);
t = t(:);
t(1) = [];
dt = t(2)-t(1);
j = sqrt(-1);
q = sqrt(qx.^2+qy.^2+qz.^2);
% for a given q, we need to integrate.
if Rh > 0
    FF = zeros(size(q));
    for i=1:numel(q)
        rh = [cos(t), sin(t), v*t];
        v0 = vnz*[-sin(t), cos(t), v*ones(length(t), 1)];
        %qvdivq = [sin(psi(i))*cos(chi(i)), sin(psi(i))*sin(chi(i)), cos(psi(i))];
        qvdivq = [qx(i), qy(i), qz(i)]/q(i);
        thh = acos(v0*qvdivq');
        qr = q(i).*sin(thh);
        %th =sin(psi(i))*sin(chi(i)-t)+v*cos(psi(i));
        %th2 =sin(psi(i))*cos(chi(i)-t)+v*t*cos(psi(i));
        %thh2 = acos(vnz*th);
        %thh - thh2
        th2 = Rh*rh*qvdivq';
        F = 2*pi*R^2*besseljc(R*qr(:)).*exp(-j*q(i)*th2)*Rh_vnz*dt;

        if N>1
            Nph = (1-exp(-N*j*q(i)*ph))./(1-exp(-j*q(i)*ph));
        else
            Nph = 1;
        end
        FF(i) = sum(F).*Nph;
    end
else
    qr = sqrt(qx.^2+qy.^2);
    FF = 2*pi*R^2*helixlength*besseljc(R*qr(:)).*sinc(qz*helixlength/2).*exp(-j*qz*helixlength/2);
    if N>1
        Nph = (1-exp(-N*j*qz*helixlength))./(1-exp(-j*qz*helixlength));
    else
        Nph = 1;
    end
    FF = FF.*Nph;
end
FF(isnan(FF)) = vol;
% k = abs(FF) > vol
% if sum(k) > 1
%     FF
% end
end